  program DateTimeOper
    !
    ! Performs an operation on date time parameters
    !
    !
    !  #[ use statements
    USE DateTimeMod
    USE Compiler_Features, only: getarg_genscat, iargc_genscat
    !  #]
    !  #[ local variables
    implicit none

    type(date_type)      :: input_date1, input_date2
    type(time_type)      :: input_time1, input_time2
    integer              :: year_tmp, ddd_tmp
    character(len=6)     :: time_tmp
    integer              :: input_hrs
    integer              :: julian1, julian2
    integer(kind=8)      :: time_secs

    integer              :: iarg,ios
    integer              :: nargs
    character(len=256)   :: arg,arg2   ! command line argument
    character(len=256)   :: input_dtg1, input_dtg2
    logical              :: calc_abs   = .false.
    logical              :: use_julian = .false.
    integer              :: oper_mode
    !  #]

    !  #[

    ! check # of arguments
    nargs = iargc_genscat()
    if ( (nargs .lt. 2) .or. (nargs .gt. 5) ) then
       print *,"Wrong nr of arguments !!"
       call usage()
    endif

    ! arguments
    iarg  = 0
    oper_mode = 0
    argloop: do while (iarg .lt. nargs)
       iarg = iarg+1
       call getarg_genscat(iarg,arg)
       if (arg(1:1) .ne. '-') call usage()
       arg2 = arg(2:)
       ! possible options are: -s  -a  -abs
       arg2_select: select case(arg2)
          !---------------------------------------------------
       case ('s')   ! subtract DTGs from command line
          oper_mode = 1
          iarg = iarg+1
          call getarg_genscat(iarg,arg)
          input_dtg1 = arg
          iarg = iarg+1
          call getarg_genscat(iarg,arg)
          input_dtg2 = arg
          !---------------------------------------------------
       case ('a')   ! add hours to DTG from command line
          oper_mode = 2
          iarg = iarg+1
          call getarg_genscat(iarg,arg)
          input_dtg1 = arg
          iarg = iarg+1
          call getarg_genscat(iarg,arg)
          read(arg,'(I5)', iostat=ios) input_hrs
          ! add a check on ios result?
          !---------------------------------------------------
       case ('c')   ! convert julian DTG from command line to YYYYMMDD
          oper_mode = 3
          iarg = iarg+1
          call getarg_genscat(iarg,arg)
          input_dtg1 = arg
          !---------------------------------------------------
       case ('cj')   ! convert YYYYMMDD from command line to julian DTG
          oper_mode = 4
          iarg = iarg+1
          call getarg_genscat(iarg,arg)
          input_dtg1 = arg
          !---------------------------------------------------
       case ('abs')   ! calculate absolute value of result
          calc_abs = .true.
          !---------------------------------------------------
       case ('j')     ! use julian dates (YYYYDDD)
          use_julian = .true.
          !---------------------------------------------------
       case default
          write(*,'(2A)') 'Invalid option ', trim(arg)
          call usage()
          !---------------------------------------------------
       end select arg2_select
    enddo argloop

    ! subtract DTG1 - DTG2
    if (oper_mode .eq. 1) then
      if (use_julian) then ! convert to YYMMDD[HHMMSS] format
        read(input_dtg1,'(I4)') year_tmp
        read(input_dtg1,'(4X,I3)') ddd_tmp
        read(input_dtg1,'(7X,A6)') time_tmp
        write(input_dtg1,'(I8,A6)') DayJulian(year_tmp,ddd_tmp), time_tmp
        read(input_dtg2,'(I4)') year_tmp
        read(input_dtg2,'(4X,I3)') ddd_tmp
        read(input_dtg2,'(7X,A6)') time_tmp
        write(input_dtg2,'(I8,A6)') DayJulian(year_tmp,ddd_tmp), time_tmp
      endif
      call string2date(input_dtg1, input_date1, input_time1)
      call string2date(input_dtg2, input_date2, input_time2)
      ! in some notations the value 24 for hours is used - correct by adding 0 seconds
      if (input_time1%hour .eq. 24) then
        call increment_datetime(input_date1,input_time1,0.)
      endif
      if (input_time2%hour .eq. 24) then
        call increment_datetime(input_date2,input_time2,0.)
      endif
      if (valid_date(input_date1) .and. valid_time(input_time1) .and. &
          valid_date(input_date2) .and. valid_time(input_time2)) then
        call ymd2julian(input_date1%year, input_date1%month, input_date1%day, julian1)
        call ymd2julian(input_date2%year, input_date2%month, input_date2%day, julian2)
        time_secs = (julian1 - julian2) * 60 * 60 * 24
        time_secs = time_secs + (input_time1%hour   - input_time2%hour  ) * 60 * 60
        time_secs = time_secs + (input_time1%minute - input_time2%minute) * 60
        ! second is stored as real, nint makes conversion to integer explicit
        time_secs = time_secs + nint(input_time1%second - input_time2%second)
        if (calc_abs) time_secs = abs(time_secs)
        if (time_secs .lt. -99999999) then
          write(*,'(A)') '-99999999'
        elseif (time_secs .gt. 999999999) then
          write(*,'(A)') '999999999'
        else
          write(*,'(I9)') time_secs
        endif
      endif
    endif

    ! add HRS to DTG1
    if (oper_mode .eq. 2) then
      if (use_julian) then ! convert to YYMMDD[HHMMSS] format
        read(input_dtg1,'(I4)') year_tmp
        read(input_dtg1,'(4X,I3)') ddd_tmp
        read(input_dtg1,'(7X,A6)') time_tmp
        write(input_dtg1,'(I8,A6)') DayJulian(year_tmp,ddd_tmp), time_tmp
      endif
      call string2date(input_dtg1, input_date1, input_time1)
      ! in some notations the value 24 for hours is used - correct by adding 0 seconds
      if (input_time1%hour .eq. 24) then
        call increment_datetime(input_date1,input_time1,0.)
      endif
      if (valid_date(input_date1) .and. valid_time(input_time1)) then
        call increment_datetime(input_date1,input_time1,60.*60.*real(input_hrs))
        if (use_julian) then ! convert to YYDDDHHMMSS format
          write(*,'(I4.4,I3.3,3I2.2)') input_date1%year, &
             JulianDay(input_date1%year, input_date1%month, input_date1%day), &
             input_time1%hour, input_time1%minute, int(input_time1%second)
        else
          write(*,'(I4.4,5I2.2)') input_date1%year, input_date1%month, input_date1%day, &
             input_time1%hour, input_time1%minute, int(input_time1%second)
        endif
      endif
    endif

    ! convert julian DTG to YYYYMMDD
    if (oper_mode .eq. 3) then
      read(input_dtg1,'(I4)') year_tmp
      read(input_dtg1,'(4X,I3)') ddd_tmp
      read(input_dtg1,'(7X,A6)') time_tmp
      write(*,'(I8,A6)') DayJulian(year_tmp,ddd_tmp), time_tmp
    endif

    ! convert YYYYMMDD to julian DTG
    if (oper_mode .eq. 4) then
      call string2date(input_dtg1, input_date1, input_time1)
      write(*,'(I4.4,I3.3,A6)') input_date1%year, &
        JulianDay(input_date1%year,input_date1%month,input_date1%day), input_dtg1(9:14)
    endif

  end program DateTimeOper
  !  #]

  subroutine usage()
    !  #[
    !
    implicit none

    write(*,*) ' '
    write(*,*) 'Usage: DateTimeOper [operation] [options]'
    write(*,*) ' '
    write(*,*) 'operations:'
    write(*,*) ' '
    write(*,*) ' -s  DTG1 DTG2 - subtract DTG2 from DTG1 and present result in seconds'
    write(*,*) ' -a  DTG1 HRS  - add HRS hours to DTG1 and present result as DTG'
    write(*,*) ' -c  DTG1      - convert DTG1 from julian format YYYYDDD[HHMMSS] into format YYYYMMDD[HHMMSS]'
    write(*,*) ' -cj DTG1      - convert DTG1 from format YYYYMMDD[HHMMSS] into julian format YYYYDDD[HHMMSS]'
    write(*,*) ' '
    write(*,*) 'options:'
    write(*,*) ' '
    write(*,*) ' -abs - present absolute value of result'
    write(*,*) ' -j   - input/output DTGs in julian format YYYYDDD[HHMMSS]'
    write(*,*) '        else, DTGs must be in format YYYYMMDD[HHMMSS]'
    write(*,*) ' '
    stop

  end subroutine usage
  !  #]
