#!/bin/ksh
#
# add local BUFR defintions for different scatterometer data to the global tables
# this is not the official way to do it, it should be done by adding local tables
# but that would make it more complicated and require that environment variable
# $ECCODES_DEFINITION_PATH is always set, which is not necessary with this method
#
# Anton Verhoef, April 2020

DEFSDIR=ecmwf/install/share/eccodes/definitions/bufr/tables/0/wmo/13

# make a backup copy of the original files
if [ ! -s ${DEFSDIR}/sequence.deforg ] ; then
  cp -p ${DEFSDIR}/sequence.def ${DEFSDIR}/sequence.deforg
fi
if [ ! -s ${DEFSDIR}/element.tableorg ] ; then
  cp -p ${DEFSDIR}/element.table ${DEFSDIR}/element.tableorg
fi

# add table D entry for CFOSAT (already submitted to WMO, operational in Nov 2020)
if [ `grep '"312034"' ${DEFSDIR}/sequence.def | wc -l` -eq 0 ] ; then
  echo '"312034" = [  301046, 301011, 301013, 301023, 008025, 201136, 004006, 201000, 312031, 312032,'  >> ${DEFSDIR}/sequence.def
  echo '               101004, 312030, 101002, 312033, 103018, 021110, 301023, 321028 ]'                >> ${DEFSDIR}/sequence.def
fi

# add local table D entry for Seawinds format with generic wind section
if [ `grep '"312223"' ${DEFSDIR}/sequence.def | wc -l` -eq 0 ] ; then
  echo '"312223" = [  301046, 301011, 301013, 301023, 008025, 201136, 004006, 201000, 005034, 006034,'  >> ${DEFSDIR}/sequence.def
  echo '               021103, 312032, 101002, 312033, 021110, 301023, 321028, 021111, 301023, 321028,' >> ${DEFSDIR}/sequence.def
  echo '               021112, 301023, 321028, 021113, 301023, 321028, 312059 ]'                        >> ${DEFSDIR}/sequence.def
fi

# add link to fix master table version 0 which is present in old ERS data
CURRDIR=`pwd`
cd ${DEFSDIR}/..
if [ ! -e 0 ] && [ ! -d 0 ] ; then
  ln -s 13 0
fi
cd ${CURRDIR}

# add local table D entries for KNMI ERS data
if [ `grep '"312201"' ${DEFSDIR}/sequence.def | wc -l` -eq 0 ] ; then
  echo '"312201" = [  301237, 101003, 301239, 011012, 011011, 021197 ]'                                >> ${DEFSDIR}/sequence.def
fi
if [ `grep '"301237"' ${DEFSDIR}/sequence.def | wc -l` -eq 0 ] ; then
  echo '"301237" = [  001007, 001012, 002021, 301011, 004004, 004005, 201138, 202131, 004006, 201000,' >> ${DEFSDIR}/sequence.def
  echo '               202000, 301023 ]'                                                               >> ${DEFSDIR}/sequence.def
fi
if [ `grep '"301239"' ${DEFSDIR}/sequence.def | wc -l` -eq 0 ] ; then
  echo '"301239" = [  002111, 002112, 021062, 021193, 021065 ]'                                        >> ${DEFSDIR}/sequence.def
fi

# add local table B entries for KNMI ERS data
# code|abbreviation|type|name|unit|scale|reference|width|crex_unit|crex_scale|crex_width
if [ `grep ^001193 ${DEFSDIR}/element.table | wc -l` -eq 0 ] ; then
  echo '001193|sigma0SimulationMethodLoc|table|SIGMA_0 SIMULATION METHOD|CODE TABLE|0|0|4|CODE TABLE|0|2'   >> ${DEFSDIR}/element.table
fi
if [ `grep ^021193 ${DEFSDIR}/element.table | wc -l` -eq 0 ] ; then
  echo '021193|noiseFigureLoc|long|NOISE FIGURE|%|0|0|7|%|0|3'                                              >> ${DEFSDIR}/element.table
fi
if [ `grep ^021197 ${DEFSDIR}/element.table | wc -l` -eq 0 ] ; then
  echo '021197|uwiProductConfidenceLoc|flag|UWI PRODUCT CONFIDENCE|FLAG TABLE|0|0|12|FLAG TABLE|0|4'        >> ${DEFSDIR}/element.table
fi
if [ `grep ^021225 ${DEFSDIR}/element.table | wc -l` -eq 0 ] ; then
  echo '021225|prescatProductConfidenceLoc|flag|PRESCAT PRODUCT CONFIDENCE|FLAG TABLE|0|0|6|FLAG TABLE|0|2' >> ${DEFSDIR}/element.table
fi
if [ `grep ^021226 ${DEFSDIR}/element.table | wc -l` -eq 0 ] ; then
  echo '021226|backscatterDistanceLoc|long|BACKSCATTER DISTANCE|Numeric|1|-4096|13|Numeric|1|4'             >> ${DEFSDIR}/element.table
fi
if [ `grep ^033213 ${DEFSDIR}/element.table | wc -l` -eq 0 ] ; then
  echo '033213|1dVarRadianceCostLoc|long|1D VAR RADIANCE COST|Numeric|1|-1000|11|Numeric|1|4'               >> ${DEFSDIR}/element.table
fi

# create empty file so that we know that the script has been executed
touch local_defs.done
