program test_get_colloc
  !  #[ documentation
  ! a generic program to test the get_colloc_from_GRIB_filelist function
  ! which is the main entrypoint for the GRIBIO fortran interface
  !
  ! TODO: add possibility to also test the get_from_GRIB_filelist
  !       fortran function from this program.
  !       (even if get_colloc_from_GRIB_filelist does have 6 calls to
  !        this function, it is an important user imterface by itselfd)
  !
  ! Written by: J. de Kloe, 2016
  !  #]
  !  #[ modules used
  use GribIO, only: init_GRIB_module, dealloc_all_grib_messages, &
       get_colloc_from_GRIB_filelist, add_to_GRIB_filelist, &
       GribVerbosityInfo, set_grib_filelist
  use Compiler_Features, only: getarg_genscat, iargc_genscat
  use numerics, only: r4_, r8_
  use lunmanager, only: get_lun, free_lun
  !  #]
  !  #[ variables
  implicit none

  integer :: nargs, current_iarg
  character(len=512) :: arg_text, line
  character(len=512) :: grib_filename, grib_filename_list
  character(len=256), dimension(:), allocatable :: grib_filenames
  integer :: nr_of_GRIB_files, fileunit, i
  real(r4_) :: lat, lon, time
  integer :: date, grib_par_id, level, level_type
  integer :: time_step, time_offset, min_analyse_distance
  logical :: use_3point_interpolation, enable_two_analyses_times, &
             do_proximity_check, logn_to_linear_conversion
  logical :: select_level, select_level_type, verbose
  real(r8_) :: interp_value
  
  ! not yet used:
  !integer :: alt_grib_par_id, derived_date, derived_hour
  !  #]
  !  #[ set defaults
  !error_flag = no_error

  grib_filename(:)          = ' '
  grib_filename_list(:)     = ' '
  !alt_grib_par_id=
  !derived_date=
  !derived_hour=
  level                     = -1
  select_level              = .false.
  level_type                = -1
  select_level_type         = .false.
  use_3point_interpolation  = .false.
  time_step                 = 3
  time_offset               = 0
  enable_two_analyses_times = .false.
  min_analyse_distance      = time_step
  do_proximity_check        = .true.
  logn_to_linear_conversion = .false.
  verbose                   = .true.
  !  #]
  !  #[ commandline handling
  nargs = iargc_genscat()
  if (nargs .lt. 5) then
     call print_usage()
     stop
  endif
  
  ! the first 5 arguments must always be present
  call getarg_genscat(1,arg_text)
  read(arg_text, *) lat
  call getarg_genscat(2,arg_text)
  read(arg_text, *) lon
  call getarg_genscat(3,arg_text)
  read(arg_text, *) time
  call getarg_genscat(4,arg_text)
  read(arg_text, *) date
  call getarg_genscat(5,arg_text)
  read(arg_text, *) grib_par_id
  
  current_iarg = 5
  do while (current_iarg .lt. nargs)
     current_iarg = current_iarg+1
     call getarg_genscat(current_iarg, arg_text)
     select case(trim(arg_text))
     !case('--alt_grib_par_id')
     !case('--derived_date')
     !case('--derived_hour')
     case('--grib_filename')
        current_iarg = current_iarg+1
        if (current_iarg .gt. nargs) then
           print *,"filename is missing after --grib_filename!"
           call print_usage()
           stop
        end if
        call getarg_genscat(current_iarg, arg_text)
        grib_filename = arg_text
     case('--grib_filename_list')
        current_iarg = current_iarg+1
        if (current_iarg .gt. nargs) then
           print *,"filename is missing after --grib_filename_list!"
           call print_usage()
           stop
        end if
        call getarg_genscat(current_iarg, arg_text)
        grib_filename_list = arg_text
     case('--level')
        select_level = .true.
        current_iarg = current_iarg+1
        if (current_iarg .gt. nargs) then
           print *,"level number is missing after --level option!"
           call print_usage()
           stop
        end if
        call getarg_genscat(current_iarg, arg_text)
        read(arg_text, *) level
     case('--select_level_type')
        select_level_type = .true.
        current_iarg = current_iarg+1
        if (current_iarg .gt. nargs) then
           print *,"level type number is missing after --level_type option!"
           call print_usage()
           stop
        end if
        call getarg_genscat(current_iarg, arg_text)
        read(arg_text, *) level_type
     case('--use_3point_interpolation')
        use_3point_interpolation = .true.
     case('--time_step')
        current_iarg = current_iarg+1
        if (current_iarg .gt. nargs) then
           print *,"time step is missing after --time_step option!"
           call print_usage()
           stop
        end if
        call getarg_genscat(current_iarg, arg_text)
        read(arg_text, *) time_step
     case('--time_offset')
        current_iarg = current_iarg+1
        if (current_iarg .gt. nargs) then
           print *,"time offset is missing after --time_offset option!"
           call print_usage()
           stop
        end if
        call getarg_genscat(current_iarg, arg_text)
        read(arg_text, *) time_offset
     case('--enable_two_analyses_times')
        enable_two_analyses_times = .true.
     case('--min_analyse_distance')
        current_iarg = current_iarg+1
        if (current_iarg .gt. nargs) then
           print *,"min_analyse_distance is missing after "//&
                "--min_analyse_distance option!"
           call print_usage()
           stop
        end if
        call getarg_genscat(current_iarg, arg_text)
        read(arg_text, *) min_analyse_distance
     case('--do_proximity_check')
        do_proximity_check = .true.
     case('--logn_to_linear_conversion')
        logn_to_linear_conversion = .true.
     case ("-h","--help")
        call print_usage()
        stop
     case('--verbose')
        verbose=.true.
     case('--silent')
        verbose=.false.
     case default
        print *,'ERROR: parsing arguments failed at argument:'
        print *,current_iarg,':',trim(arg_text)
        call print_usage()
        stop
     end select
  end do

  ! some sanity checks
  if ( (len_trim(grib_filename)      .eq. 0) .and. &
       (len_trim(grib_filename_list) .eq. 0)       ) then
     print *,'ERROR: one of the options: '
     print *,'           --grib_filename <filename>'
     print *,'           --grib_filename_list <filename>'
     print *,'is required, but both seem absent!'
     stop
  end if

  if ( (len_trim(grib_filename)      .gt. 0) .and. &
       (len_trim(grib_filename_list) .gt. 0)       ) then
     print *,'ERROR: one of the options: '
     print *,'           --grib_filename <filename>'
     print *,'           --grib_filename_list <filename>'
     print *,'is required, but both seem present!'
     stop
  end if
  
  if (verbose) then
     print *,"============================="
     print *,"REALACC(6)"
     print *,"Selected settings:"
     print *,'verbose                   = ', verbose
     print *,'grib_filename             = ', trim(grib_filename)
     print *,'grib_filename_list        = ', trim(grib_filename_list)
     print *,"lat                       = ", lat, "[deg]"
     print *,"lon                       = ", lon, "[deg]"
     print *,"time                      = ", time, "[s]"
     print *,"date                      = ", date
     print *,"grib_par_id               = ", grib_par_id
     !print *,"alt_grib_par_id           = ", alt_grib_par_id
     !print *,"derived_date              = ", derived_date
     !print *,"derived_hour              = ", derived_hour
     if (select_level) &
          print *,"level                     = ", level
     if (select_level_type) &
          print *,"level_type                = ", level_type
     print *,"use_3point_interpolation  = ", use_3point_interpolation
     print *,"time_step                 = ", time_step, "h"
     print *,"time_offset               = ", time_offset, "h"
     print *,"enable_two_analyses_times = ", enable_two_analyses_times
     print *,"min_analyse_distance      = ", min_analyse_distance, "h"
     print *,"do_proximity_check        = ", do_proximity_check
     print *,"logn_to_linear_conversion = ", logn_to_linear_conversion
     print *,"ENDREALACC"
  end if
  !  #]
  !  #[ the actual work
  if (verbose) then
     call init_GRIB_module(Verbosity=GribVerbosityInfo)
  else
     call init_GRIB_module()
  end if

  if (len_trim(grib_filename) .gt. 0) then
     call add_to_GRIB_filelist(grib_filename)
  end if
  if (len_trim(grib_filename_list) .gt. 0) then
     ! first call nr. of input files
     !character(len=512), dimension(:), allocatable :: grib_filename_list
     fileunit = get_lun()
     nr_of_GRIB_files = 0
     open(unit=fileunit, file=trim(grib_filename_list),&
          status='old',action='read')
     do
        read(fileunit,*,end=111) line
        if (len_trim(line) .gt. 0) nr_of_GRIB_files = nr_of_GRIB_files+1
     end do
     ! jump point for end-of-file
111  continue
     rewind(fileunit)

     allocate(grib_filenames(nr_of_GRIB_files))
     
     ! now read the file names
     i=0
     do
        read(fileunit,*,end=222) line
        if (len_trim(line) .gt. 0) then
           i = i+1
           grib_filenames(i) = trim(line)
        end if
     end do
     ! jump point for end-of-file
222  continue

     call free_lun(fileunit)

     !print *,'nr_of_GRIB_files = ', nr_of_GRIB_files
     !do i=1,nr_of_GRIB_files
     !   print *,trim(grib_filenames(i))
     !end do

     call set_GRIB_filelist(grib_filenames, nr_of_GRIB_files)
  end if
  
  !grib_filename_list
  
  interp_value = get_colloc_from_GRIB_filelist( &
       lat, lon, time, date, grib_par_id, &
       set_time_step             = time_step, &
       set_time_offset           = time_offset, &
       set_min_analyse_distance  = min_analyse_distance, &
       set_proximity_check       = do_proximity_check, &
       enable_two_analyses_times = enable_two_analyses_times, &
       use_3point_interpolation  = use_3point_interpolation, &
       logn_to_linear_conversion = logn_to_linear_conversion)

  print *,"REALACC(6)"
  print *,'Result: interp_value = ', interp_value
  print *,"ENDREALACC"
  
!if (select_level) then ... level ...
!if (select_level_type) then ... level_type ...
  
  call dealloc_all_GRIB_messages()

   if (allocated(grib_filenames)) deallocate(grib_filenames)
  
  !  #]
contains
  !----------------------
  subroutine print_usage()
    !  #[ print some help text
    print *,' '
    print *,'Usage: '
    print *,' '
    print *,'   test_get_colloc <lat> <lon> <time> <date> <grib_par_id>'//&
         ' [input] [options]'
    print *,' '
    ! required parameters
    print *,'in which the required parameters are:'
    print *,'   <lat> is a float giving the latitude in degrees'
    print *,'   <lon> is a float giving the longitude in degrees'
    print *,'   <time> is an integer giving the time of day in seconds'
    print *,'   <date> is an string giving the date as HHMMSS'
    print *,'   <grib_par_id> is an integer to indicate the grib parameter'
    print *,' '
    print *,'and in which [input] is only one of the following two options:'
    print *,' '
    print *,'   --grib_filename <filename>'
    print *,'     selects the grib file to be read'
    print *,' '
    print *,'   --grib_filename_list <filename>'
    print *,'     selects data for the given level'
    print *,'     [default is to not select on this]'
    print *,' '
    ! optional parameters
    print *,'and in which [options] may be empty or any combination of:'
    print *,' '
    ! not used for now
    !print *,'   --alt_grib_par_id <grib_par_id> '
    !print *,'     this allows the code to fall back to an alternative grib'
    !print *,'     parameter, in case the primary grib_par_id cannot be found'
    !print *,'     [default is to no use it]'
    !print *,' '
    ! not used for now
    !print *,'   --derived_date <date>'
    !print *,'     selects on validity date (i.e. date of an+fc)'
    !print *,'     [default is to not select on this]'
    !print *,' '
    ! not used for now
    !print *,'   --derived_hour <hour>'
    !print *,'     selects on validity hour (i.e. hour of an+fc)'
    !print *,'     [default is to not select on this]'
    !print *,' '
    print *,'   --level <lvl>'
    print *,'     selects data for the given level'
    print *,'     [default is to not select on this]'
    print *,' '
    print *,'   --level_type <lvltype>'
    print *,'     selects data for the given level type'
    print *,'     [default is to not select on this]'
    print *,' '
    print *,'   --use_3point_interpolation'
    print *,'     switch on 3 point time interpolation'
    print *,'     [default is .false.]'
    print *,' '
    print *,'   --time_step <step>'
    print *,'     manually define the time step [in hours] between forecasts'
    print *,'     to a number that differs from the default'
    print *,'     [default is 3 hours]'
    print *,'     '
    print *,'   --time_offset <offset>'
    print *,'     allows to add an offset to the analysis time [in hours]'
    print *,'     [I dont remember what the purpose is for this one]'
    print *,'     [default is 0 hours]'
    print *,'     '
    print *,'   --enable_two_analyses_times'
    print *,'     allow grib messages from several model runs to be present'
    print *,'     in the provided list of grib files, and tries to'
    print *,'     automatically select the most suitable fields to be used'
    print *,'     [default is .false.]'
    print *,'     '
    print *,'   --min_analyse_distance <dist>'
    print *,'     allow the data to be closer to an analyse used for'
    print *,'     interpolation then time_step hours. Normally you dont want'
    print *,'     to make sure that the same cell-data was not used to produce'
    print *,'     this analyse in the first place'
    print *,'     (since this would introduce unwanted dependencies)'
    print *,'     [default is to use the current time_step value]'
    print *,'     '
    print *,'   --do_proximity_check'
    print *,'     apply the analyse distance check as mentioned above or not'
    print *,'     [default is .true.]'
    print *,'     '
    print *,'   --logn_to_linear_conversion'
    print *,'     make sure exp(val) in stead of val is used before doing'
    print *,'     any interpolations, and report exp(val) in stead of val'
    print *,'     [default is .false.]'
    print *,'     '
    print *,'   --verbose'
    print *,'     switch to verbose mode to print a lot of internal'
    print *,'     details during the grib file handling.'
    print *,'     [default is verbose]'
    print *,'     '
    print *,'   --silent'
    print *,'     switch on silent mode, and only print the result'
    print *,'     of the requested interpolation, or the missing'
    print *,'     number if no result could be obtained.'
    print *,'     [default is verbose]'

    stop

  end subroutine print_usage
    !  #]
end program test_get_colloc
