program test_read_BUFR1

  ! a small program to test the BUFR Fortran interface

  use BufrIO, only: set_BUFR_fileattributes,get_BUFR_nr_of_messages, &
                    open_BUFR_file,get_BUFR_message,close_BUFR_file, &
                    save_BUFR_message,bufr_file_attr_data,bufr_data_type
  use numerics, only: missing_indicator_real
  
  implicit none

  character(len=256) :: filename_bufr
  type(bufr_file_attr_data) :: bufr_file_attributes
  type(bufr_data_type) :: tbd
  integer :: error_flag,nr_of_messages,i,pos_lon,pos_lat
  real    :: lat,lon,min_lat,max_lat,min_lon,max_lon

  ! test file containing one ERS BUFR message
  filename_bufr = "./testfile.bufr"

  print '(A)', " open BUFR file"
  print '(2A)'," file name = ",trim(filename_bufr)

  call set_BUFR_fileattributes(filename_bufr,bufr_file_attributes,error_flag)
  if (error_flag .ne. 0) stop
  call open_BUFR_file(bufr_file_attributes,error_flag)
  if (error_flag .ne. 0) stop

  nr_of_messages = get_BUFR_nr_of_messages(bufr_file_attributes)
  print '(A,I4)'," number of BUFR messages in file   = ",nr_of_messages

  ! read first BUFR message
  call get_BUFR_message(bufr_file_attributes,1,tbd,error_flag)
  print '(A,I4)'," number of subsets in BUFR message = ",tbd%nsubsets

  ! search first descriptors from descriptor list denoting latitude and longitude
  pos_lat = -1
  pos_lon = -1
  do i = 1,tbd%nelements
    if (tbd%expanded_descr(i) .eq. 5002 .and. pos_lat .eq. -1) pos_lat = i
    if (tbd%expanded_descr(i) .eq. 6002 .and. pos_lon .eq. -1) pos_lon = i
  enddo
  print '(A,I4)'," latitude  position = ",pos_lat
  print '(A,I4)'," longitude position = ",pos_lon

  min_lat =   90.0
  max_lat =  -90.0
  min_lon =  180.0
  max_lon = -180.0
  do i = 1,tbd%nsubsets
    lat = real(tbd%values(tbd%nelements*(i-1)+pos_lat))
    lon = real(tbd%values(tbd%nelements*(i-1)+pos_lon))
    if (lat .lt. missing_indicator_real*0.99 .and. lon .lt. missing_indicator_real*0.99) then
      if (lat .gt. max_lat) max_lat = lat
      if (lat .lt. min_lat) min_lat = lat
      if (lon .gt. max_lon) max_lon = lon
      if (lon .lt. min_lon) min_lon = lon
    endif
  enddo
  print '(A,2F8.3)'," latitude  range: ",min_lat,max_lat
  print '(A,2F8.3)'," longitude range: ",min_lon,max_lon

  ! close input file
  call close_BUFR_file(bufr_file_attributes,error_flag)

  ! open bufr file for writing
  filename_bufr = "./testwrite.bufr"

  print '(A)', " open BUFR file for writing"
  print '(2A)'," file name = ",trim(filename_bufr)

  call set_BUFR_fileattributes(filename_bufr,bufr_file_attributes,error_flag)
  if (error_flag .ne. 0) stop
  call open_BUFR_file(bufr_file_attributes,error_flag,writemode = .true.)
  if (error_flag .ne. 0) stop

  ! save BUFR message that was just read to the output file
  call save_BUFR_message(bufr_file_attributes,tbd,error_flag)
  if (error_flag .ne. 0) stop

  ! close output file
  call close_BUFR_file(bufr_file_attributes,error_flag)
  if (error_flag .ne. 0) stop

  print '(A)', " BUFR file was written successfully"

end program test_read_BUFR1
