program test_read_GRIB1

  ! a small program to test GRIB fortran interface

  use GribIO, only: xget_from_GRIB_file, xgrib_get_date,&
       xgrib_get_derived_date, xgrib_get_derived_time, xgrib_get_time, &
       par_id_10u, par_id_10v, init_grib_module, dealloc_all_grib_messages
  use numerics, only: r4_
  
  implicit none

  character(len=256) :: filename_grib
  real    :: dummy
  real    :: lat, lon, step
  real    :: lat_start, lat_end, lon_start, lon_end
  integer :: time, date
  integer :: lat_cnt, lon_cnt, num_lat_steps, num_lon_steps

  call init_GRIB_module()

  dummy = 0.

  ! original name of this testfile was: NWP_20031111_12_12_lowres.grib
  filename_grib = "./testfile.grib"

  print '(A)', "open GRIB edition 1 file"
  print '(2A)',"file name = ",trim(filename_grib)
  date = nint(xget_from_GRIB_file(filename_grib, xgrib_get_date, dummy, par_id_10v))
  time = nint(xget_from_GRIB_file(filename_grib, xgrib_get_time, dummy, par_id_10u))
  print '(A,I11)',"date of grib field =         ",date
  print '(A,I11)',"time of grib field =         ",time

  date = nint(xget_from_GRIB_file(filename_grib, xgrib_get_derived_date, dummy, par_id_10v))
  time = nint(xget_from_GRIB_file(filename_grib, xgrib_get_derived_time, dummy, par_id_10u))

  ! require the numerical output to be reproducible
  ! with 4 digits accuracy (as tested by the difftool)
  print *,'REALACC(4)'
  print '(A,I11)',"derived date of grib field = ",date
  print '(A,I11)',"derived time of grib field = ",time
  print '(A)', " "
  print '(A)', "    lat    lon      10u      10v    speed"

  step      =  0.5
  lat_start = 54.0 ! position on the North Sea near the Netherlands
  lat_end   = 56.0
  lon_start =  4.0
  lon_end   =  6.0
  num_lat_steps = nint((lat_end - lat_start)/step)
  num_lon_steps = nint((lon_end - lon_start)/step)

  ! loop over lat and lon values
  do lat_cnt = 0,num_lat_steps-1
    lat = lat_start + real(lat_cnt)*step
    do lon_cnt = 0,num_lon_steps-1
      lon = lon_start + real(lon_cnt)*step

      ! write GRIB data
      print '(2F7.2,3F9.3)', lat, lon, &
        real(xget_from_GRIB_file(filename_grib, lat, lon, par_id_10u),r4_), &
        real(xget_from_GRIB_file(filename_grib, lat, lon, par_id_10v),r4_), &
        sqrt(real(xget_from_GRIB_file(filename_grib, lat, lon, par_id_10u),r4_)**2 + &
             real(xget_from_GRIB_file(filename_grib, lat, lon, par_id_10v),r4_)**2)
    enddo
  enddo

  print *,"ENDREALACC"
  
  call dealloc_all_GRIB_messages()

  call init_GRIB_module()

  dummy = 0.

  ! this file was converted using tools/grib_set -s edition=2 testfile.grib testfile.grib2
  filename_grib = "./testfile.grib2"

  print '(A)', " "
  print '(A)', "open GRIB edition 2 file"
  print '(2A)',"file name = ",trim(filename_grib)
  date = nint(xget_from_GRIB_file(filename_grib, xgrib_get_date, dummy, par_id_10v))
  time = nint(xget_from_GRIB_file(filename_grib, xgrib_get_time, dummy, par_id_10u))
  print '(A,I11)',"date of grib field =         ",date
  print '(A,I11)',"time of grib field =         ",time
  print '(A)', " "
  print '(A)', "End of tests"

  call dealloc_all_GRIB_messages()

end program test_read_GRIB1
