module HDF5Mod
!
!  #[ Description:
!
!   This module contains interface routines for calling HDF GROUP hdf5
!   routines in C from Fortran.
!
!  #]
!  #[
! Current Code Owner:  Anton Verhoef
!
! Code Description:
!     Language:           Fortran 90
!     Software Standards: "European Standards for Writing and
!     Documenting Exchangeable Fortran 90 Code"
!
! This software was developed within the context of the EUMETSAT Satellite
! Application Facility on Numerical Weather Prediction (NWP SAF), under the
! Cooperation Agreement dated 16 December, 2003, between EUMETSAT and the
! Met Office, UK, by one or more partners within the NWP SAF. The partners
! in the NWP SAF are the Met Office, ECMWF, KNMI and Meteo France.
!
! Copyright 2008, EUMETSAT, All Rights Reserved.
!
!     Unit Name:             HDF5Mod
!
!     Created on:            Mar. 2010
!
!     Last Modified on:      $Date$
!
!     Modifications Log:
!
! $Id$
!
!
!  #]
!  #[ modules used:
!
  use numerics,only: r4_
!
!  #]
!
  implicit none
!
!  #[ public declarations
!
  integer,public,parameter :: hdf5_verbosity_error  = 0
  integer,public,parameter :: hdf5_verbosity_warn   = 1
  integer,public,parameter :: hdf5_verbosity_report = 2

  integer,public,save      :: hdf5_verbosity = hdf5_verbosity_error
!
!  #]
!  #[ private declarations
!
!
!  #]
!  #[ interfaces
!
  interface
    function h5f_open_c(f_name,verbosity)
      character(len=*),intent(in) :: f_name
      integer,intent(in)          :: verbosity
      integer                     :: h5f_open_c
    end function h5f_open_c
  end interface

  interface
    function h5f_create_c(f_name,verbosity)
      character(len=*),intent(in) :: f_name
      integer,intent(in)          :: verbosity
      integer                     :: h5f_create_c
    end function h5f_create_c
  end interface

  interface
    function h5f_close_c(f_id)
      integer,intent(in) :: f_id
      integer            :: h5f_close_c
    end function h5f_close_c
  end interface

  interface
    function h5g_open_c(f_id,g_name)
      integer,intent(in)          :: f_id
      character(len=*),intent(in) :: g_name
      integer                     :: h5g_open_c
    end function h5g_open_c
  end interface

  interface
    function h5g_create_c(f_id,g_name)
      integer,intent(in)          :: f_id
      character(len=*),intent(in) :: g_name
      integer                     :: h5g_create_c
    end function h5g_create_c
  end interface

  interface
    function h5g_close_c(g_id)
      integer,intent(in) :: g_id
      integer            :: h5g_close_c
    end function h5g_close_c
  end interface

  interface
    function h5a_open_name_c(l_id,a_name)
      integer,intent(in)          :: l_id
      character(len=*),intent(in) :: a_name
      integer                     :: h5a_open_name_c
    end function h5a_open_name_c
  end interface

  interface
    function h5a_create_c(l_id,a_name,t_id,s_id)
      integer,intent(in)          :: l_id
      character(len=*),intent(in) :: a_name
      integer,intent(in)          :: t_id
      integer,intent(in)          :: s_id
      integer                     :: h5a_create_c
    end function h5a_create_c
  end interface

  interface
    function h5a_get_type_c(a_id)
      integer,intent(in) :: a_id
      integer            :: h5a_get_type_c
    end function h5a_get_type_c
  end interface

  interface
    function h5a_read_char_c(a_id,t_id,sdata)
      integer,intent(in)           :: a_id
      integer,intent(in)           :: t_id
      character(len=*),intent(out) :: sdata
      integer                      :: h5a_read_char_c
    end function h5a_read_char_c
  end interface

  interface
    function h5a_read_int_c(a_id,t_id,sdata)
      integer,intent(in)           :: a_id
      integer,intent(in)           :: t_id
      integer,intent(out)          :: sdata
      integer                      :: h5a_read_int_c
    end function h5a_read_int_c
  end interface

  interface
    function h5a_read_flt_c(a_id,t_id,sdata)
      integer,intent(in)           :: a_id
      integer,intent(in)           :: t_id
      real,   intent(out)          :: sdata
      integer                      :: h5a_read_flt_c
    end function h5a_read_flt_c
  end interface

  interface
    function h5a_write_char_c(a_id,t_id,sdata)
      integer,intent(in)          :: a_id
      integer,intent(in)          :: t_id
      character(len=*),intent(in) :: sdata
      integer                     :: h5a_write_char_c
    end function h5a_write_char_c
  end interface

  interface
    function h5a_close_c(a_id)
      integer,intent(in) :: a_id
      integer            :: h5a_close_c
    end function h5a_close_c
  end interface

  interface
    function h5d_open_c(l_id,d_name)
      integer,intent(in)          :: l_id
      character(len=*),intent(in) :: d_name
      integer                     :: h5d_open_c
    end function h5d_open_c
  end interface

  interface
    function h5d_create_int_c(l_id,d_name,s_id)
      integer,intent(in)          :: l_id
      character(len=*),intent(in) :: d_name
      integer,intent(in)          :: s_id
      integer                     :: h5d_create_int_c
    end function h5d_create_int_c
  end interface

  interface
    function h5d_create_float_c(l_id,d_name,s_id)
      integer,intent(in)          :: l_id
      character(len=*),intent(in) :: d_name
      integer,intent(in)          :: s_id
      integer                     :: h5d_create_float_c
    end function h5d_create_float_c
  end interface

  interface
    function h5d_create_char_c(l_id,d_name,t_id,s_id)
      integer,intent(in)          :: l_id
      character(len=*),intent(in) :: d_name
      integer,intent(in)          :: t_id
      integer,intent(in)          :: s_id
      integer                     :: h5d_create_char_c
    end function h5d_create_char_c
  end interface

  interface
    function h5d_close_c(d_id)
      integer,intent(in) :: d_id
      integer            :: h5d_close_c
    end function h5d_close_c
  end interface

  interface
    function h5d_get_type_c(d_id)
      integer,intent(in) :: d_id
      integer            :: h5d_get_type_c
    end function h5d_get_type_c
  end interface

  interface
    function h5d_read_int_c(d_id,idata)
      integer,intent(in)               :: d_id
      integer,dimension(*),intent(out) :: idata
      integer                          :: h5d_read_int_c
    end function h5d_read_int_c
  end interface

  interface
    function h5d_read_float_c(d_id,rdata,idata)
      use numerics,only: r4_
      integer,intent(in)                 :: d_id
      real(r4_),dimension(*),intent(out) :: rdata
      integer,dimension(*),intent(out)   :: idata
      integer                            :: h5d_read_float_c
    end function h5d_read_float_c
  end interface

  interface
    function h5d_read_char_c(d_id,t_id,sdata)
      integer,intent(in)                        :: d_id
      integer,intent(in)                        :: t_id
      character(len=*),dimension(*),intent(out) :: sdata
      integer                                   :: h5d_read_char_c
    end function h5d_read_char_c
  end interface

  interface
    function h5d_read_part_int_c(d_id,ms_id,fs_id,idata)
      integer,intent(in)               :: d_id
      integer,intent(in)               :: ms_id
      integer,intent(in)               :: fs_id
      integer,dimension(*),intent(out) :: idata
      integer                          :: h5d_read_part_int_c
    end function h5d_read_part_int_c
  end interface

  interface
    function h5d_read_part_float_c(d_id,ms_id,fs_id,rdata)
      use numerics,only: r4_
      integer,intent(in)                 :: d_id
      integer,intent(in)                 :: ms_id
      integer,intent(in)                 :: fs_id
      real(r4_),dimension(*),intent(out) :: rdata
      integer                            :: h5d_read_part_float_c
    end function h5d_read_part_float_c
  end interface

  interface
    function h5d_write_int_c(d_id,idata)
      integer,intent(in)              :: d_id
      integer,dimension(*),intent(in) :: idata
      integer                         :: h5d_write_int_c
    end function h5d_write_int_c
  end interface

  interface
    function h5d_write_float_c(d_id,rdata)
      use numerics,only: r4_
      integer,intent(in)                :: d_id
      real(r4_),dimension(*),intent(in) :: rdata
      integer                           :: h5d_write_float_c
    end function h5d_write_float_c
  end interface

  interface
    function h5d_write_char_c(d_id,t_id,sdata)
      integer,intent(in)                       :: d_id
      integer,intent(in)                       :: t_id
      character(len=*),dimension(*),intent(in) :: sdata
      integer                                  :: h5d_write_char_c
    end function h5d_write_char_c
  end interface

  interface
    function h5d_get_space_c(d_id)
      integer,intent(in) :: d_id
      integer            :: h5d_get_space_c
    end function h5d_get_space_c
  end interface

  interface
    function h5s_get_select_npoints_c(s_id)
      integer,intent(in) :: s_id
      integer            :: h5s_get_select_npoints_c
    end function h5s_get_select_npoints_c
  end interface

  interface
    function h5s_select_hyperslab_c(s_id,start,stride,count)
      integer,intent(in)              :: s_id
      integer,dimension(*),intent(in) :: start
      integer,dimension(*),intent(in) :: stride
      integer,dimension(*),intent(in) :: count
      integer                         :: h5s_select_hyperslab_c
    end function h5s_select_hyperslab_c
  end interface

  interface
    function h5s_create_simple_c(rank,dims)
      integer,intent(in)              :: rank
      integer,dimension(*),intent(in) :: dims
      integer                         :: h5s_create_simple_c
    end function h5s_create_simple_c
  end interface

  interface
    function h5s_close_c(s_id)
      integer,intent(in) :: s_id
      integer            :: h5s_close_c
    end function h5s_close_c
  end interface

  interface
    function h5t_get_size_c(t_id)
      integer,intent(in) :: t_id
      integer            :: h5t_get_size_c
    end function h5t_get_size_c
  end interface

  interface
    function h5t_set_string_size_c(len)
      integer,intent(in) :: len
      integer            :: h5t_set_string_size_c
    end function h5t_set_string_size_c
  end interface

  interface
    function h5t_close_c(t_id)
      integer,intent(in) :: t_id
      integer            :: h5t_close_c
    end function h5t_close_c
  end interface

!
!  #]
!
contains

  subroutine init_hdf5_module(verbosity)
    !  #[
    !
    ! Description: initialise HDF5 module and set verbosity
    !
    integer,optional,intent(in) :: verbosity

    if (present(verbosity)) then
      hdf5_verbosity = verbosity
    endif

  end subroutine init_hdf5_module
    !  #]

  function h5f_open(f_name) result(f_id)
    !  #[
    !
    ! Description: open HDF5 file for reading
    !
    character(len=*),intent(in) :: f_name
    integer                     :: f_id

    f_id = h5f_open_c(trim(f_name) // achar(0),hdf5_verbosity)

  end function h5f_open
    !  #]

  function h5f_create(f_name) result(f_id)
    !  #[
    !
    ! Description: open HDF5 file for writing
    !
    character(len=*),intent(in) :: f_name
    integer                     :: f_id

    f_id = h5f_create_c(trim(f_name) // achar(0),hdf5_verbosity)

  end function h5f_create
    !  #]

  function h5f_close(f_id) result(ierr)
    !  #[
    !
    ! Description: close HDF5 file
    !
    integer,intent(in) :: f_id
    integer            :: ierr

    ierr = h5f_close_c(f_id)

  end function h5f_close
    !  #]

  function h5g_open(f_id,g_name) result(g_id)
    !  #[
    !
    ! Description: open HDF5 group in file
    !
    integer,intent(in)          :: f_id
    character(len=*),intent(in) :: g_name
    integer                     :: g_id

    g_id = h5g_open_c(f_id,trim(g_name) // achar(0))

  end function h5g_open
    !  #]

  function h5g_create(f_id,g_name) result(g_id)
    !  #[
    !
    ! Description: create HDF5 group in file
    !
    integer,intent(in)          :: f_id
    character(len=*),intent(in) :: g_name
    integer                     :: g_id

    g_id = h5g_create_c(f_id,trim(g_name) // achar(0))

  end function h5g_create
    !  #]

  function h5g_close(g_id) result(ierr)
    !  #[
    !
    ! Description: close HDF5 group
    !
    integer,intent(in) :: g_id
    integer            :: ierr

    ierr = h5g_close_c(g_id)

  end function h5g_close
    !  #]

  function h5d_open(l_id,d_name) result(d_id)
    !  #[
    !
    ! Description: open HDF5 dataset in location (file or group)
    !
    integer,intent(in)          :: l_id
    character(len=*),intent(in) :: d_name
    integer                     :: d_id

    d_id = h5d_open_c(l_id,trim(d_name) // achar(0))

  end function h5d_open
    !  #]

  function h5d_close(d_id) result(ierr)
    !  #[
    !
    ! Description: close HDF5 dataset
    !
    integer,intent(in) :: d_id
    integer            :: ierr

    ierr = h5d_close_c(d_id)

  end function h5d_close
    !  #]

  function h5d_get_space(d_id) result(s_id)
    !  #[
    !
    ! Description: get space of HDF5 dataset
    !
    integer,intent(in) :: d_id
    integer            :: s_id

    s_id = h5d_get_space_c(d_id)

  end function h5d_get_space
    !  #]

  function h5d_get_npoints(d_id) result(npoints)
    !  #[
    !
    ! Description: obtain number of points in a dataset
    !
    integer,intent(in) :: d_id
    integer            :: npoints

    integer :: s_id

    npoints = -1
    s_id = h5d_get_space_c(d_id)
    if (s_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Space of dataset',d_id,' could not be found'
      endif
      return
    endif

    npoints = h5s_get_select_npoints_c(s_id)
    if (npoints .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Number of datapoints of dataspace',s_id,' could not be found'
      endif
      return
    endif

    if (h5s_close_c(s_id) .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataspace with s_id',s_id,' could not be closed'
      endif
      npoints = -1
      return
    endif

  end function h5d_get_npoints
    !  #]

  function h5d_read_int(d_id,idata) result(ierr)
    !  #[
    !
    ! Description: read integer HDF5 dataset
    !
    integer,intent(in)               :: d_id
    integer,dimension(:),intent(out) :: idata
    integer                          :: ierr

    ierr = h5d_read_int_c(d_id,idata)

  end function h5d_read_int
    !  #]

  function h5d_read_int_2d(d_id,idata, d1, d2) result(ierr)
    !  #[
    !
    ! Description: read integer HDF5 dataset
    !
    integer,intent(in)                    :: d_id, d1, d2
    integer,dimension(d1, d2),intent(out) :: idata
    integer                               :: ierr

    ierr = h5d_read_int_c(d_id,idata)

  end function h5d_read_int_2d
    !  #]

  function h5d_read_int_3d(d_id,idata, d1, d2, d3) result(ierr)
    !  #[
    !
    ! Description: read integer HDF5 dataset
    !
    integer,intent(in)                        :: d_id, d1, d2, d3
    integer,dimension(d1, d2, d3),intent(out) :: idata
    integer                                   :: ierr

    ierr = h5d_read_int_c(d_id,idata)

  end function h5d_read_int_3d
    !  #]

  function h5d_read_float(d_id,rdata) result(ierr)
    !  #[
    !
    ! Description: read float HDF5 dataset
    !
    integer,intent(in)                 :: d_id
    real(r4_),dimension(:),intent(out) :: rdata
    integer                            :: ierr

    integer,dimension(:),allocatable   :: idata

    allocate(idata(size(rdata)))
    ierr = h5d_read_float_c(d_id,rdata,idata)

    ! status 33 denotes that we succeeded to read the data as integer values
    ! so copy the integer values into the real array
    if (ierr .eq. 33) then
      rdata = real(idata)
      ierr = 0
    endif

    if (allocated(idata)) deallocate(idata)

  end function h5d_read_float
    !  #]

  function h5d_read_float_2d(d_id,rdata, d1, d2) result(ierr)
    !  #[
    !
    ! Description: read float HDF5 dataset
    !
    integer,intent(in)                   :: d_id, d1, d2
    real(r4_),dimension(:,:),intent(out) :: rdata
    integer                              :: ierr

    integer,dimension(:,:),allocatable   :: idata

    allocate(idata(d1, d2))
    ierr = h5d_read_float_c(d_id,rdata,idata)

    ! status 33 denotes that we succeeded to read the data as integer values
    ! so copy the integer values into the real array
    if (ierr .eq. 33) then
      rdata = real(idata)
      ierr = 0
    endif

    if (allocated(idata)) deallocate(idata)

  end function h5d_read_float_2d
     !  #]

  function h5d_read_float_3d(d_id,rdata, d1, d2, d3) result(ierr)
    !  #[
    !
    ! Description: read float HDF5 dataset
    !
    integer,intent(in)                     :: d_id, d1, d2, d3
    real(r4_),dimension(:,:,:),intent(out) :: rdata
    integer                                :: ierr

    integer,dimension(:,:,:),allocatable   :: idata

    allocate(idata(d1, d2, d3))
    ierr = h5d_read_float_c(d_id,rdata,idata)

    ! status 33 denotes that we succeeded to read the data as integer values
    ! so copy the integer values into the real array
    if (ierr .eq. 33) then
      rdata = real(idata)
      ierr = 0
    endif

    if (allocated(idata)) deallocate(idata)

  end function h5d_read_float_3d
     !  #]

  function h5d_read_string_2d_cfo(d_id,sdata) result(ierr)
    !  #[
    !
    ! Description: obtain contents of string dataset d_id
    !
    integer,intent(in)                          :: d_id
    character(len=1),dimension(:,:),intent(out) :: sdata
    integer                                     :: ierr

    integer :: t_id

    ierr = -1
    t_id = h5d_get_type_c(d_id)
    if (t_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Could not get type of dataset',d_id
      endif
      return
    endif

    ierr = h5d_read_char_c(d_id,t_id,sdata)

  end function h5d_read_string_2d_cfo
    !  #]

  function h5d_read_string(d_id,sdata) result(ierr)
    !  #[
    !
    ! Description: obtain contents of string dataset d_id
    !
    integer,intent(in)                        :: d_id
    character(len=*),dimension(:),intent(out) :: sdata
    integer                                   :: ierr

    integer :: t_id,csize,i,j

    ierr = -1
    t_id = h5d_get_type_c(d_id)
    if (t_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Could not get type of dataset',d_id
      endif
      return
    endif

    csize = h5t_get_size_c(t_id)

    if (csize .le. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Could not get size of type',t_id
      endif
      return
    endif

    ! check if sdata is properly defined
    if (csize .ne. len(sdata)) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Size of hdf5 dataset is',csize,', len of character variable is',len(sdata)
      endif
      return
    endif

    ierr = h5d_read_char_c(d_id,t_id,sdata)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataset',d_id,' could not be read'
      endif
      return
    endif

    ierr = h5t_close_c(t_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Type with t_id',t_id,' could not be closed'
      endif
      return
    endif

    ! replace \0 characters by spaces
    do i = 1,size(sdata)
      do j = 1,len(sdata)
        if (ichar(sdata(i)(j:j)) .eq. 0) sdata(i)(j:j) = ' '
      enddo
    enddo

  end function h5d_read_string
    !  #]




  function h5d_read_part_int_1d(d_id,ms_id,fs_id,idata) result(ierr)
    !  #[
    !
    ! Description: read part of integer HDF5 dataset
    !
    integer,intent(in)               :: d_id
    integer,intent(in)               :: ms_id
    integer,intent(in)               :: fs_id
    integer,dimension(:),intent(out) :: idata
    integer                          :: ierr

    ierr = h5d_read_part_int_c(d_id,ms_id,fs_id,idata)

  end function h5d_read_part_int_1d
    !  #]

  function h5d_read_part_int_2d(d_id,ms_id,fs_id,idata) result(ierr)
    !  #[
    !
    ! Description: read part of integer HDF5 dataset
    !
    integer,intent(in)                 :: d_id
    integer,intent(in)                 :: ms_id
    integer,intent(in)                 :: fs_id
    integer,dimension(:,:),intent(out) :: idata
    integer                            :: ierr

    ierr = h5d_read_part_int_c(d_id,ms_id,fs_id,idata)

  end function h5d_read_part_int_2d
    !  #]

  function h5d_read_part_float_1d(d_id,ms_id,fs_id,rdata) result(ierr)
    !  #[
    !
    ! Description: read part of float HDF5 dataset
    !
    integer,intent(in)                 :: d_id
    integer,intent(in)                 :: ms_id
    integer,intent(in)                 :: fs_id
    real(r4_),dimension(:),intent(out) :: rdata
    integer                            :: ierr

    ierr = h5d_read_part_float_c(d_id,ms_id,fs_id,rdata)

  end function h5d_read_part_float_1d
    !  #]

  function h5d_read_part_float_2d(d_id,ms_id,fs_id,rdata) result(ierr)
    !  #[
    !
    ! Description: read part of float HDF5 dataset
    !
    integer,intent(in)                   :: d_id
    integer,intent(in)                   :: ms_id
    integer,intent(in)                   :: fs_id
    real(r4_),dimension(:,:),intent(out) :: rdata
    integer                              :: ierr

    ierr = h5d_read_part_float_c(d_id,ms_id,fs_id,rdata)

  end function h5d_read_part_float_2d
    !  #]

  subroutine h5d_open_and_read_int(l_id,d_name,d_size,idata,ierr)
    !  #[
    !
    ! Description: open HDF5 dataset in location (file or group) and
    !   read integer HDF5 dataset
    !
    integer,intent(in)                    :: l_id
    character(len=*),intent(in)           :: d_name
    integer,intent(in)                    :: d_size
    integer,dimension(d_size),intent(out) :: idata
    integer,intent(out)                   :: ierr

    integer :: d_id

    ierr = -1
    d_id = h5d_open(l_id,d_name)
    if (d_id .gt. 0) then
      ierr = h5d_read_int(d_id,idata)
      if (ierr .eq. 0) then
        ierr = h5d_close(d_id)
      endif
    endif

  end subroutine h5d_open_and_read_int
    !  #]

  subroutine h5d_open_and_read_float(l_id,d_name,d_size,rdata,ierr)
    !  #[
    !
    ! Description: open HDF5 dataset in location (file or group) and
    !   read float HDF5 dataset
    !
    integer,intent(in)                 :: l_id
    character(len=*),intent(in)        :: d_name
    integer,intent(in)                 :: d_size
    real,dimension(d_size),intent(out) :: rdata
    integer,intent(out)                :: ierr

    integer :: d_id

    ierr = -1
    d_id = h5d_open(l_id,d_name)
    if (d_id .gt. 0) then
      ierr = h5d_read_float(d_id,rdata)
      if (ierr .eq. 0) then
        ierr = h5d_close(d_id)
      endif
    endif

  end subroutine h5d_open_and_read_float
    !  #]

  subroutine h5d_open_and_read_string(l_id,d_name,d_size,sdata,ierr)
    !  #[
    !
    ! Description: open HDF5 dataset in location (file or group) and
    !   read string HDF5 dataset
    !
    integer,intent(in)                             :: l_id
    character(len=*),intent(in)                    :: d_name
    integer,intent(in)                             :: d_size
    character(len=*),dimension(d_size),intent(out) :: sdata
    integer,intent(out)                            :: ierr

    integer :: d_id

    ierr = -1
    d_id = h5d_open(l_id,d_name)
    if (d_id .gt. 0) then
      ierr = h5d_read_string(d_id,sdata)
      if (ierr .eq. 0) then
        ierr = h5d_close(d_id)
      endif
    endif

  end subroutine h5d_open_and_read_string
    !  #]

  subroutine h5d_open_and_extract_sds_int(l_id,d_name,dim1,index,idata,ierr)
    !  #[
    !
    ! Description: open HDF5 dataset in location (file or group) and
    !   extract int HDF5 sub-dataset
    !
    integer,intent(in)                  :: l_id
    character(len=*),intent(in)         :: d_name
    integer,intent(in)                  :: dim1
    integer,intent(in)                  :: index
    integer,dimension(dim1),intent(out) :: idata
    integer,intent(out)                 :: ierr

    integer :: d_id
    integer :: filespace                  ! file dataspace identifier
    integer :: memspace                   ! memory dataspace identifier
    integer :: memrank = 1                ! dataset rank in memory

    integer, dimension(2) :: offset_in    ! hyperslab offset in the file
    integer, dimension(2) :: stride_in    ! hyperslab stride in the file
    integer, dimension(2) :: count_in     ! size of the hyperslab in the file
    integer, dimension(1) :: offset_out   ! hyperslab offset in memory
    integer, dimension(1) :: stride_out   ! hyperslab stride in memory
    integer, dimension(1) :: count_out    ! size of the hyperslab in memory

    ierr = -1
    d_id = h5d_open(l_id,d_name)
    if (d_id .lt. 0) then
      ierr = d_id
      return
    endif

    offset_in(1)  = 0
    offset_in(2)  = index - 1
    stride_in(1)  = 1
    stride_in(2)  = 1
    count_in(1)   = dim1
    count_in(2)   = 1
    offset_out(1) = 0
    stride_out(1) = 1
    count_out(1)  = dim1

    ! get dataset's file dataspace identifier
    filespace = h5d_get_space(d_id)
    if (filespace .lt. 0) then
      ierr = filespace
      return
    endif

    ! select hyperslab in the file dataset
    ierr = h5s_select_hyperslab(filespace,offset_in,stride_in,count_in)

    ! create memory dataspace
    memspace = h5s_create_simple(memrank,count_out)

    ! select hyperslab in memory dataspace
    ierr = h5s_select_hyperslab(memspace,offset_out,stride_out,count_out)

    ! read data from hyperslab in the file into the hyperslab in memory
    ierr = h5d_read_part_int_1d(d_id,memspace,filespace,idata)

    ! close the memoryspace
    ierr = h5s_close(memspace)

    ! close the file dataspace for the dataset and the dataset
    ierr = h5s_close(filespace)
    ierr = h5d_close(d_id)

  end subroutine h5d_open_and_extract_sds_int
    !  #]

  subroutine h5d_open_and_extract_sds_int_2d(l_id,d_name,dim2,dim1,index,idata,ierr)
    !  #[
    !
    ! Description: open HDF5 dataset in location (file or group) and
    !   extract 2d int HDF5 sub-dataset
    !
    integer,intent(in)                       :: l_id
    character(len=*),intent(in)              :: d_name
    integer,intent(in)                       :: dim2
    integer,intent(in)                       :: dim1
    integer,intent(in)                       :: index
    integer,dimension(dim2,dim1),intent(out) :: idata
    integer,intent(out)                      :: ierr

    integer :: d_id
    integer :: filespace                 ! file dataspace identifier
    integer :: memspace                  ! memory dataspace identifier
    integer :: memrank = 2               ! dataset rank in memory

    integer,dimension(3) :: offset_in    ! hyperslab offset in the file
    integer,dimension(3) :: stride_in    ! hyperslab stride in the file
    integer,dimension(3) :: count_in     ! size of the hyperslab in the file
    integer,dimension(2) :: offset_out   ! hyperslab offset in memory
    integer,dimension(2) :: stride_out   ! hyperslab stride in memory
    integer,dimension(2) :: count_out    ! size of the hyperslab in memory

    ierr = -1
    d_id = h5d_open(l_id,d_name)
    if (d_id .lt. 0) then
      ierr = d_id
      return
    endif

    offset_in(1)  = 0
    offset_in(2)  = 0
    offset_in(3)  = index - 1
    stride_in(1)  = 1
    stride_in(2)  = 1
    stride_in(3)  = 1
    count_in(1)   = dim2
    count_in(2)   = dim1
    count_in(3)   = 1
    offset_out(1) = 0
    offset_out(2) = 0
    stride_out(1) = 1
    stride_out(2) = 1
    count_out(1)  = dim2
    count_out(2)  = dim1

    ! get dataset's file dataspace identifier
    filespace = h5d_get_space(d_id)
    if (filespace .lt. 0) then
      ierr = filespace
      return
    endif

    ! select hyperslab in the file dataset
    ierr = h5s_select_hyperslab(filespace,offset_in,stride_in,count_in)

    ! create memory dataspace
    memspace = h5s_create_simple(memrank,count_out)

    ! select hyperslab in memory dataspace
    ierr = h5s_select_hyperslab(memspace,offset_out,stride_out,count_out)

    ! read data from hyperslab in the file into the hyperslab in memory
    ierr = h5d_read_part_int_2d(d_id,memspace,filespace,idata)

    ! close the memoryspace
    ierr = h5s_close(memspace)

    ! close the file dataspace for the dataset and the dataset
    ierr = h5s_close(filespace)
    ierr = h5d_close(d_id)

  end subroutine h5d_open_and_extract_sds_int_2d
    !  #]

  subroutine h5d_open_and_extract_sds_float(l_id,d_name,dim1,index,rdata,ierr)
    !  #[
    !
    ! Description: open HDF5 dataset in location (file or group) and
    !   extract float HDF5 sub-dataset
    !
    integer,intent(in)               :: l_id
    character(len=*),intent(in)      :: d_name
    integer,intent(in)               :: dim1
    integer,intent(in)               :: index
    real,dimension(dim1),intent(out) :: rdata
    integer,intent(out)              :: ierr

    integer :: d_id
    integer :: filespace                  ! file dataspace identifier
    integer :: memspace                   ! memory dataspace identifier
    integer :: memrank = 1                ! dataset rank in memory

    integer, dimension(2) :: offset_in    ! hyperslab offset in the file
    integer, dimension(2) :: stride_in    ! hyperslab stride in the file
    integer, dimension(2) :: count_in     ! size of the hyperslab in the file
    integer, dimension(1) :: offset_out   ! hyperslab offset in memory
    integer, dimension(1) :: stride_out   ! hyperslab stride in memory
    integer, dimension(1) :: count_out    ! size of the hyperslab in memory

    ierr = -1
    d_id = h5d_open(l_id,d_name)
    if (d_id .lt. 0) then
      ierr = d_id
      return
    endif

    offset_in(1)  = 0
    offset_in(2)  = index - 1
    stride_in(1)  = 1
    stride_in(2)  = 1
    count_in(1)   = dim1
    count_in(2)   = 1
    offset_out(1) = 0
    stride_out(1) = 1
    count_out(1)  = dim1

    ! get dataset's file dataspace identifier
    filespace = h5d_get_space(d_id)
    if (filespace .lt. 0) then
      ierr = filespace
      return
    endif

    ! select hyperslab in the file dataset
    ierr = h5s_select_hyperslab(filespace,offset_in,stride_in,count_in)

    ! create memory dataspace
    memspace = h5s_create_simple(memrank,count_out)

    ! select hyperslab in memory dataspace
    ierr = h5s_select_hyperslab(memspace,offset_out,stride_out,count_out)

    ! read data from hyperslab in the file into the hyperslab in memory
    ierr = h5d_read_part_float_1d(d_id,memspace,filespace,rdata)

    ! close the memoryspace
    ierr = h5s_close(memspace)

    ! close the file dataspace for the dataset and the dataset
    ierr = h5s_close(filespace)
    ierr = h5d_close(d_id)

  end subroutine h5d_open_and_extract_sds_float
    !  #]

  subroutine h5d_open_and_extract_sds_float_2d(l_id,d_name,dim2,dim1,index,rdata,ierr)
    !  #[
    !
    ! Description: open HDF5 dataset in location (file or group) and
    !   extract 2d float HDF5 sub-dataset
    !
    integer,intent(in)                    :: l_id
    character(len=*),intent(in)           :: d_name
    integer,intent(in)                    :: dim2
    integer,intent(in)                    :: dim1
    integer,intent(in)                    :: index
    real,dimension(dim2,dim1),intent(out) :: rdata
    integer,intent(out)                   :: ierr

    integer :: d_id
    integer :: filespace                 ! file dataspace identifier
    integer :: memspace                  ! memory dataspace identifier
    integer :: memrank = 2               ! dataset rank in memory

    integer,dimension(3) :: offset_in    ! hyperslab offset in the file
    integer,dimension(3) :: stride_in    ! hyperslab stride in the file
    integer,dimension(3) :: count_in     ! size of the hyperslab in the file
    integer,dimension(2) :: offset_out   ! hyperslab offset in memory
    integer,dimension(2) :: stride_out   ! hyperslab stride in memory
    integer,dimension(2) :: count_out    ! size of the hyperslab in memory

    ierr = -1
    d_id = h5d_open(l_id,d_name)
    if (d_id .lt. 0) then
      ierr = d_id
      return
    endif

    offset_in(1)  = 0
    offset_in(2)  = 0
    offset_in(3)  = index - 1
    stride_in(1)  = 1
    stride_in(2)  = 1
    stride_in(3)  = 1
    count_in(1)   = dim2
    count_in(2)   = dim1
    count_in(3)   = 1
    offset_out(1) = 0
    offset_out(2) = 0
    stride_out(1) = 1
    stride_out(2) = 1
    count_out(1)  = dim2
    count_out(2)  = dim1

    ! get dataset's file dataspace identifier
    filespace = h5d_get_space(d_id)
    if (filespace .lt. 0) then
      ierr = filespace
      return
    endif

    ! select hyperslab in the file dataset
    ierr = h5s_select_hyperslab(filespace,offset_in,stride_in,count_in)

    ! create memory dataspace
    memspace = h5s_create_simple(memrank,count_out)

    ! select hyperslab in memory dataspace
    ierr = h5s_select_hyperslab(memspace,offset_out,stride_out,count_out)

    ! read data from hyperslab in the file into the hyperslab in memory
    ierr = h5d_read_part_float_2d(d_id,memspace,filespace,rdata)

    ! close the memoryspace
    ierr = h5s_close(memspace)

    ! close the file dataspace for the dataset and the dataset
    ierr = h5s_close(filespace)
    ierr = h5d_close(d_id)

  end subroutine h5d_open_and_extract_sds_float_2d
    !  #]

  function h5d_write_int_1d(l_id,d_name,idata) result(ierr)
    !  #[
    !
    ! Description: write integer 1-dimensional array in location l_id
    !   location can be a file or group
    !
    integer,intent(in)              :: l_id
    character(len=*),intent(in)     :: d_name
    integer,dimension(:),intent(in) :: idata
    integer                         :: ierr

    integer :: s_id,d_id
    integer,dimension(1) :: dims

    ierr = -1
    dims(1) = size(idata)

    s_id = h5s_create_simple_c(1,dims)
    if (s_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataspace could not be created'
      endif
      return
    endif

    d_id = h5d_create_int_c(l_id,trim(d_name) // achar(0),s_id)
    if (d_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataset ',trim(d_name),' could not be created'
      endif
      return
    endif

    ierr = h5d_write_int_c(d_id,idata)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataset ',trim(d_name),' could not be written'
      endif
      return
    endif

    ierr = h5d_close_c(d_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataset with d_id',d_id,' could not be closed'
      endif
      return
    endif

    ierr = h5s_close_c(s_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataspace with s_id',s_id,' could not be closed'
      endif
      return
    endif

  end function h5d_write_int_1d
    !  #]

  function h5d_write_int_2d(l_id,d_name,idata) result(ierr)
    !  #[
    !
    ! Description: write integer 2-dimensional array in location l_id
    !   location can be a file or group
    !
    integer,intent(in)                :: l_id
    character(len=*),intent(in)       :: d_name
    integer,dimension(:,:),intent(in) :: idata
    integer                           :: ierr

    integer :: s_id,d_id
    integer,dimension(2) :: dims

    ierr = -1
    dims(1) = size(idata,1)
    dims(2) = size(idata,2)

    s_id = h5s_create_simple_c(2,dims)
    if (s_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataspace could not be created'
      endif
      return
    endif

    d_id = h5d_create_int_c(l_id,trim(d_name) // achar(0),s_id)
    if (d_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataset ',trim(d_name),' could not be created'
      endif
      return
    endif

    ierr = h5d_write_int_c(d_id,idata)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataset ',trim(d_name),' could not be written'
      endif
      return
    endif

    ierr = h5d_close_c(d_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataset with d_id',d_id,' could not be closed'
      endif
      return
    endif

    ierr = h5s_close_c(s_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataspace with s_id',s_id,' could not be closed'
      endif
      return
    endif

  end function h5d_write_int_2d
    !  #]

  function h5d_write_int_3d(l_id,d_name,idata) result(ierr)
    !  #[
    !
    ! Description: write integer 3-dimensional array in location l_id
    !   location can be a file or group
    !
    integer,intent(in)                :: l_id
    character(len=*),intent(in)       :: d_name
    integer,dimension(:,:,:),intent(in) :: idata
    integer                           :: ierr

    integer :: s_id,d_id
    integer,dimension(3) :: dims

    ierr = -1
    dims(1) = size(idata,1)
    dims(2) = size(idata,2)
    dims(3) = size(idata,3)

    s_id = h5s_create_simple_c(3,dims)
    if (s_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataspace could not be created'
      endif
      return
    endif

    d_id = h5d_create_int_c(l_id,trim(d_name) // achar(0),s_id)
    if (d_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataset ',trim(d_name),' could not be created'
      endif
      return
    endif

    ierr = h5d_write_int_c(d_id,idata)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataset ',trim(d_name),' could not be written'
      endif
      return
    endif

    ierr = h5d_close_c(d_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataset with d_id',d_id,' could not be closed'
      endif
      return
    endif

    ierr = h5s_close_c(s_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataspace with s_id',s_id,' could not be closed'
      endif
      return
    endif

  end function h5d_write_int_3d
    !  #]

  function h5d_write_float_1d(l_id,d_name,rdata) result(ierr)
    !  #[
    !
    ! Description: write float 1-dimensional array in location l_id
    !   location can be a file or group
    !
    integer,intent(in)                :: l_id
    character(len=*),intent(in)       :: d_name
    real(r4_),dimension(:),intent(in) :: rdata
    integer                           :: ierr

    integer :: s_id,d_id
    integer,dimension(1) :: dims

    ierr = -1
    dims(1) = size(rdata)

    s_id = h5s_create_simple_c(1,dims)
    if (s_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataspace could not be created'
      endif
      return
    endif

    d_id = h5d_create_float_c(l_id,trim(d_name) // achar(0),s_id)
    if (d_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataset ',trim(d_name),' could not be created'
      endif
      return
    endif

    ierr = h5d_write_float_c(d_id,rdata)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataset ',trim(d_name),' could not be written'
      endif
      return
    endif

    ierr = h5d_close_c(d_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataset with d_id',d_id,' could not be closed'
      endif
      return
    endif

    ierr = h5s_close_c(s_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataspace with s_id',s_id,' could not be closed'
      endif
      return
    endif

  end function h5d_write_float_1d
    !  #]

  function h5d_write_float_2d(l_id,d_name,rdata) result(ierr)
    !  #[
    !
    ! Description: write float 2-dimensional array in location l_id
    !   location can be a file or group
    !
    integer,intent(in)                  :: l_id
    character(len=*),intent(in)         :: d_name
    real(r4_),dimension(:,:),intent(in) :: rdata
    integer                             :: ierr

    integer :: s_id,d_id
    integer,dimension(2) :: dims

    ierr = -1
    dims(1) = size(rdata,1)
    dims(2) = size(rdata,2)

    s_id = h5s_create_simple_c(2,dims)
    if (s_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataspace could not be created'
      endif
      return
    endif

    d_id = h5d_create_float_c(l_id,trim(d_name) // achar(0),s_id)
    if (d_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataset ',trim(d_name),' could not be created'
      endif
      return
    endif

    ierr = h5d_write_float_c(d_id,rdata)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataset ',trim(d_name),' could not be written'
      endif
      return
    endif

    ierr = h5d_close_c(d_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataset with d_id',d_id,' could not be closed'
      endif
      return
    endif

    ierr = h5s_close_c(s_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataspace with s_id',s_id,' could not be closed'
      endif
      return
    endif

  end function h5d_write_float_2d
    !  #]

  function h5d_write_float_3d(l_id,d_name,rdata) result(ierr)
    !  #[
    !
    ! Description: write float 3-dimensional array in location l_id
    !   location can be a file or group
    !
    integer,intent(in)                  :: l_id
    character(len=*),intent(in)         :: d_name
    real(r4_),dimension(:,:,:),intent(in) :: rdata
    integer                             :: ierr

    integer :: s_id,d_id
    integer,dimension(3) :: dims

    ierr = -1
    dims(1) = size(rdata,1)
    dims(2) = size(rdata,2)
    dims(3) = size(rdata,3)

    s_id = h5s_create_simple_c(3,dims)
    if (s_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataspace could not be created'
      endif
      return
    endif

    d_id = h5d_create_float_c(l_id,trim(d_name) // achar(0),s_id)
    if (d_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataset ',trim(d_name),' could not be created'
      endif
      return
    endif

    ierr = h5d_write_float_c(d_id,rdata)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataset ',trim(d_name),' could not be written'
      endif
      return
    endif

    ierr = h5d_close_c(d_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataset with d_id',d_id,' could not be closed'
      endif
      return
    endif

    ierr = h5s_close_c(s_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataspace with s_id',s_id,' could not be closed'
      endif
      return
    endif

  end function h5d_write_float_3d
    !  #]

  function h5d_write_string_1d(l_id,d_name,sdata) result(ierr)
    !  #[
    !
    ! Description: write string 1-dimensional array in location l_id
    !   location can be a file or group
    !
    integer,intent(in)                       :: l_id
    character(len=*),intent(in)              :: d_name
    character(len=*),dimension(:),intent(in) :: sdata
    integer                                  :: ierr

    integer :: s_id,t_id,d_id
    integer,dimension(1) :: dims

    ierr = -1
    dims(1) = size(sdata)

    s_id = h5s_create_simple_c(1,dims)
    if (s_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataspace could not be created'
      endif
      return
    endif

    t_id = h5t_set_string_size_c(len(sdata))
    if (t_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'String size could not be set'
      endif
      return
    endif

    d_id = h5d_create_char_c(l_id,trim(d_name) // achar(0),t_id,s_id);
    if (d_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataset ',trim(d_name),' could not be created'
      endif
      return
    endif

    ierr = h5d_write_char_c(d_id,t_id,sdata)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataset ',trim(d_name),' could not be written'
      endif
      return
    endif

    ierr = h5d_close_c(d_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataset with d_id',d_id,' could not be closed'
      endif
      return
    endif

    ierr = h5t_close_c(t_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Type with t_id',t_id,' could not be closed'
      endif
      return
    endif

    ierr = h5s_close_c(s_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataspace with s_id',s_id,' could not be closed'
      endif
      return
    endif

  end function h5d_write_string_1d
    !  #]

  function h5a_get_string(l_id,a_name,adata) result(ierr)
    !  #[
    !
    ! Description: obtain contents of string attribute in location l_id
    !   location can be a file or group
    !
    integer,intent(in)           :: l_id
    character(len=*),intent(in)  :: a_name
    character(len=*),intent(out) :: adata
    integer                      :: ierr

    integer :: a_id,t_id,i

    ierr = -1
    adata = ' '

    ! files converted from HDF4 to HDF5 may contain attribute names with extension _GLOSDS
    ! if the name is not found, try if it exists with or without this extension
    a_id = h5a_open_name_c(l_id,trim(a_name) // achar(0))
    if (a_id .lt. 0) then
      if (index(a_name,'_GLOSDS') .gt. 0) then
        a_id = h5a_open_name_c(l_id,a_name(1:index(a_name,'_GLOSDS')-1) // achar(0))
      else
        a_id = h5a_open_name_c(l_id,trim(a_name) // '_GLOSDS' // achar(0))
      endif
      if (a_id .lt. 0) then
        if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
          write(*,*) 'Location with l_id',l_id,' does not contain attribute ',trim(a_name)
        endif
        return
      endif
    endif

    t_id = h5a_get_type_c(a_id)
    if (t_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Could not get type of attribute',trim(a_name)
      endif
      return
    endif

    ierr = h5a_read_char_c(a_id,t_id,adata)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Attribute',trim(a_name),' could not be read'
      endif
      return
    endif

    ierr = h5t_close_c(t_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Type with t_id',t_id,' could not be closed'
      endif
      return
    endif

    ierr = h5a_close_c(a_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Attribute with a_id',a_id,' could not be closed'
      endif
      return
    endif

    ! replace \0 characters by spaces
    do i = 1,len(adata)
      if (ichar(adata(i:i)) .eq. 0) adata(i:i) = ' '
    enddo

  end function h5a_get_string
    !  #]

  function h5a_get_int(l_id,a_name,adata) result(ierr)
    !  #[
    !
    ! Description: obtain contents of integer attribute in location l_id
    !   location can be a file or group
    !
    integer,intent(in)           :: l_id
    character(len=*),intent(in)  :: a_name
    integer,         intent(out) :: adata
    integer                      :: ierr

    integer :: a_id,t_id

    ierr = -1

    ! files converted from HDF4 to HDF5 may contain attribute names with extension _GLOSDS
    ! if the name is not found, try if it exists with or without this extension
    a_id = h5a_open_name_c(l_id,trim(a_name) // achar(0))
    if (a_id .lt. 0) then
      if (index(a_name,'_GLOSDS') .gt. 0) then
        a_id = h5a_open_name_c(l_id,a_name(1:index(a_name,'_GLOSDS')-1) // achar(0))
      else
        a_id = h5a_open_name_c(l_id,trim(a_name) // '_GLOSDS' // achar(0))
      endif
      if (a_id .lt. 0) then
        if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
          write(*,*) 'Location with l_id',l_id,' does not contain attribute ',trim(a_name)
        endif
        return
      endif
    endif

    t_id = h5a_get_type_c(a_id)
    if (t_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Could not get type of attribute',trim(a_name)
      endif
      return
    endif

    ierr = h5a_read_int_c(a_id,t_id,adata)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Attribute',trim(a_name),' could not be read'
      endif
      return
    endif

    ierr = h5t_close_c(t_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Type with t_id',t_id,' could not be closed'
      endif
      return
    endif

    ierr = h5a_close_c(a_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Attribute with a_id',a_id,' could not be closed'
      endif
      return
    endif

  end function h5a_get_int
    !  #]

  function h5a_get_flt(l_id,a_name,adata) result(ierr)
    !  #[
    !
    ! Description: obtain contents of float attribute in location l_id
    !   location can be a file or group
    !
    integer,intent(in)           :: l_id
    character(len=*),intent(in)  :: a_name
    real,            intent(out) :: adata
    integer                      :: ierr

    integer :: a_id,t_id

    ierr = -1

    ! files converted from HDF4 to HDF5 may contain attribute names with extension _GLOSDS
    ! if the name is not found, try if it exists with or without this extension
    a_id = h5a_open_name_c(l_id,trim(a_name) // achar(0))
    if (a_id .lt. 0) then
      if (index(a_name,'_GLOSDS') .gt. 0) then
        a_id = h5a_open_name_c(l_id,a_name(1:index(a_name,'_GLOSDS')-1) // achar(0))
      else
        a_id = h5a_open_name_c(l_id,trim(a_name) // '_GLOSDS' // achar(0))
      endif
      if (a_id .lt. 0) then
        if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
          write(*,*) 'Location with l_id',l_id,' does not contain attribute ',trim(a_name)
        endif
        return
      endif
    endif

    t_id = h5a_get_type_c(a_id)
    if (t_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Could not get type of attribute',trim(a_name)
      endif
      return
    endif

    ierr = h5a_read_flt_c(a_id,t_id,adata)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Attribute',trim(a_name),' could not be read'
      endif
      return
    endif

    ierr = h5t_close_c(t_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Type with t_id',t_id,' could not be closed'
      endif
      return
    endif

    ierr = h5a_close_c(a_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Attribute with a_id',a_id,' could not be closed'
      endif
      return
    endif

  end function h5a_get_flt
    !  #]

  function h5a_write_string(l_id,a_name,adata) result(ierr)
    !  #[
    !
    ! Description: write string attribute in location l_id
    !   location can be a file or group
    !
    integer,intent(in)          :: l_id
    character(len=*),intent(in) :: a_name
    character(len=*),intent(in) :: adata
    integer                     :: ierr

    integer :: s_id,t_id,a_id
    integer :: strsize
    integer,dimension(1) :: dims

    ierr = -1
    dims(1) = 1

    s_id = h5s_create_simple_c(1,dims)
    if (s_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataspace could not be created'
      endif
      return
    endif

    ! minimum attribute size 1
    strsize = len(trim(adata))
    if (strsize .lt. 1) strsize = 1
    t_id = h5t_set_string_size_c(strsize)
    if (t_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'String size could not be set'
      endif
      return
    endif

    a_id = h5a_create_c(l_id,trim(a_name) // achar(0),t_id,s_id);
    if (a_id .lt. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Attribute ',trim(a_name),' could not be created'
      endif
      return
    endif

    ierr = h5a_write_char_c(a_id,t_id,trim(adata) // achar(0));
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Attribute ',trim(a_name),' could not be written'
      endif
      return
    endif

    ierr = h5a_close_c(a_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Attribute with a_id',a_id,' could not be closed'
      endif
      return
    endif

    ierr = h5t_close_c(t_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Type with t_id',t_id,' could not be closed'
      endif
      return
    endif

    ierr = h5s_close_c(s_id)
    if (ierr .ne. 0) then
      if (hdf5_verbosity .ge. hdf5_verbosity_warn) then
        write(*,*) 'Dataspace with s_id',s_id,' could not be closed'
      endif
      return
    endif

  end function h5a_write_string
    !  #]

  function h5s_create_simple(rank,dims) result(s_id)
    !  #[
    !
    ! Description: create a new simple dataspace
    !
    integer,intent(in)              :: rank
    integer,dimension(*),intent(in) :: dims
    integer                         :: s_id

    s_id = h5s_create_simple_c(rank,dims)

  end function h5s_create_simple
    !  #]

  function h5s_select_hyperslab(s_id,start,stride,count) result(ierr)
    !  #[
    !
    ! Description: select a HDF5 hyperslab region
    !
    integer,intent(in)              :: s_id
    integer,dimension(*),intent(in) :: start
    integer,dimension(*),intent(in) :: stride
    integer,dimension(*),intent(in) :: count
    integer                         :: ierr

    ierr = h5s_select_hyperslab_c(s_id,start,stride,count)

  end function h5s_select_hyperslab
    !  #]

  function h5s_close(s_id) result(ierr)
    !  #[
    !
    ! Description: close HDF5 dataspace
    !
    integer,intent(in) :: s_id
    integer            :: ierr

    ierr = h5s_close_c(s_id)

  end function h5s_close
    !  #]

end module HDF5Mod
