program TestHDF5
!
! Example program to read a small HDF5 test file using the interface routines
! and the HDFGROUP software package.
!
  use HDF5Mod

  implicit none

  integer            :: i,ierr,npoints
  integer            :: f_id,g_id,d_id
  character(len=256) :: f_name,g_name,d_name

  integer,dimension(:),allocatable           :: idata
  integer,dimension(:,:),allocatable         :: idata2
  real,dimension(:),allocatable              :: rdata
  real,dimension(:,:),allocatable            :: rdata2
  character(len=12),dimension(:),allocatable :: sdata

  f_name = 'deflate.h5'
  g_name = '/'
  d_name = 'Dataset1'

  ! open hdf5 file for reading
  f_id = h5f_open(f_name)
  if (f_id .lt. 0) then
    write(*,*) 'File ',trim(f_name),' could not be opened successfully'
    stop
  endif
  write(*,*) 'Successfully opened file ',trim(f_name),' with f_id',f_id

  ! open dataset directly by concatenating group name and dataset name
  d_id = h5d_open(f_id,trim(g_name) // trim(d_name))
  if (d_id .lt. 0) then
    write(*,*) 'File with f_id',f_id,' does not contain dataset ',trim(g_name) // trim(d_name)
    stop
  endif
  write(*,*) 'Successfully opened dataset ',trim(g_name) // trim(d_name),' with d_id',d_id

  ierr = h5d_close(d_id)
  if (ierr .ne. 0) then
    write(*,*) 'Dataset with d_id',d_id,' could not be closed successfully'
    stop
  endif
  write(*,*) 'Successfully closed dataset with d_id',d_id

  ! open dataset by opening group first and then dataset
  g_id = h5g_open(f_id,g_name)
  if (g_id .lt. 0) then
    write(*,*) 'File with f_id',f_id,' does not contain group ',trim(g_name)
    stop
  endif
  write(*,*) 'Successfully opened group ',trim(g_name),' with g_id',g_id

  d_id = h5d_open(g_id,d_name)
  if (d_id .lt. 0) then
    write(*,*) 'Group with g_id',g_id,' does not contain dataset ',trim(d_name)
    stop
  endif
  write(*,*) 'Successfully opened dataset ',trim(d_name),' with d_id',d_id

  npoints = h5d_get_npoints(d_id)
  if (npoints .lt. 0) then
    write(*,*) 'Number of datapoints of dataset',d_id,' could not be found'
    stop
  endif
  write(*,*) 'Number of datapoints of dataset',d_id,' is',npoints

  allocate(idata(npoints))

  ierr = h5d_read_int(d_id,idata)
  if (ierr .ne. 0) then
    write(*,*) 'Dataset',d_id,' could not be read'
    stop
  endif

  write(*,*) 'First data values are:'
  write(*,'(10I8)') (idata(i),i=1,20)
  if (allocated(idata)) deallocate(idata)

  ierr = h5d_close(d_id)
  if (ierr .ne. 0) then
    write(*,*) 'Dataset with d_id',d_id,' could not be closed successfully'
    stop
  endif
  write(*,*) 'Successfully closed dataset with d_id',d_id

  ierr = h5g_close(g_id)
  if (ierr .ne. 0) then
    write(*,*) 'Group with g_id',g_id,' could not be closed successfully'
    stop
  endif
  write(*,*) 'Successfully closed group with g_id',g_id

  ! close hdf5 file
  ierr = h5f_close(f_id)
  if (ierr .ne. 0) then
    write(*,*) 'File with f_id',f_id,' could not be closed successfully'
    stop
  endif
  write(*,*) 'Successfully closed file with f_id',f_id
  write(*,*) ' '
  write(*,*) 'End of file reading tests in TestHDF5'
  write(*,*) ' '

  ! open hdf5 file for writing
  f_name = 'testfile.h5'
  f_id = h5f_create(f_name)
  if (f_id .lt. 0) then
    write(*,*) 'File ',trim(f_name),' could not be created successfully'
    stop
  endif
  write(*,*) 'Successfully opened file ',trim(f_name),' with f_id',f_id

  ! create group in hdf5 file
  g_name = 'Group1'
  g_id = h5g_create(f_id,g_name)
  if (g_id .lt. 0) then
    write(*,*) 'Group ',trim(g_name),' could not be created successfully'
    stop
  endif
  write(*,*) 'Successfully created group ',trim(g_name),' with g_id',g_id

  ! write attribute
  ierr = h5a_write_string(g_id,'Attribute1','Test attribute')
  if (ierr .ne. 0) then
    write(*,*) 'Attribute Attribute1 could not be written successfully'
    stop
  endif
  write(*,*) 'Successfully wrote Attribute Attribute1 in group',g_id

  ! fill one dimensional integer array with dummy values
  allocate(idata(20))
  do i = 1,20
    idata(i) = i - 10
  enddo

  ! write array
  ierr = h5d_write_int_1d(g_id,'Dataset1_int_1d',idata)
  if (ierr .ne. 0) then
    write(*,*) 'Dataset Dataset1_int_1d could not be written successfully'
    stop
  endif
  write(*,*) 'Successfully wrote Dataset Dataset1_int_1d in group',g_id

  if (allocated(idata)) deallocate(idata)

  ! fill two dimensional integer array with dummy values
  allocate(idata2(2,10))
  do i = 1,10
    idata2(1,i) = i - 10
    idata2(2,i) = i + 10
  enddo

  ! write array
  ierr = h5d_write_int_2d(g_id,'Dataset2_int_2d',idata2)
  if (ierr .ne. 0) then
    write(*,*) 'Dataset Dataset2_int_2d could not be written successfully'
    stop
  endif
  write(*,*) 'Successfully wrote Dataset Dataset2_int_2d in group',g_id

  if (allocated(idata2)) deallocate(idata2)

  ! fill one dimensional float array with dummy values
  allocate(rdata(20))
  do i = 1,20
    rdata(i) = real(i) - 10.5
  enddo

  ! write array
  ierr = h5d_write_float_1d(g_id,'Dataset3_float_1d',rdata)
  if (ierr .ne. 0) then
    write(*,*) 'Dataset Dataset3_float_1d could not be written successfully'
    stop
  endif
  write(*,*) 'Successfully wrote Dataset Dataset3_float_1d in group',g_id

  if (allocated(rdata)) deallocate(rdata)

  ! fill two dimensional float array with dummy values
  allocate(rdata2(2,10))
  do i = 1,10
    rdata2(1,i) = real(i) - 10.5
    rdata2(2,i) = real(i) + 10.5
  enddo

  ! write array
  ierr = h5d_write_float_2d(g_id,'Dataset4_float_2d',rdata2)
  if (ierr .ne. 0) then
    write(*,*) 'Dataset Dataset4_float_2d could not be written successfully'
    stop
  endif
  write(*,*) 'Successfully wrote Dataset Dataset4_float_2d in group',g_id

  if (allocated(rdata2)) deallocate(rdata2)

  ! fill one dimensional string array with dummy values
  allocate(sdata(4))
  sdata(1) = 'Test 1'
  sdata(2) = 'Test 2'
  sdata(3) = 'Test 3'
  sdata(4) = 'Test 4'

  ! write array
  ierr = h5d_write_string_1d(g_id,'Dataset5_string_1d',sdata)
  if (ierr .ne. 0) then
    write(*,*) 'Dataset Dataset5_string_1d could not be written successfully'
    stop
  endif
  write(*,*) 'Successfully wrote Dataset Dataset5_string_1d in group',g_id

  if (allocated(sdata)) deallocate(sdata)

  ! close group
  ierr = h5g_close(g_id)
  if (ierr .ne. 0) then
    write(*,*) 'Group with g_id',g_id,' could not be closed successfully'
    stop
  endif
  write(*,*) 'Successfully closed group with g_id',g_id

  ! close hdf5 file
  ierr = h5f_close(f_id)
  if (ierr .ne. 0) then
    write(*,*) 'File with f_id',f_id,' could not be closed successfully'
    stop
  endif
  write(*,*) 'Successfully closed file with f_id',f_id
  write(*,*) ' '
  write(*,*) 'End of file writing tests in TestHDF5'
  write(*,*) ' '
  write(*,*) 'A HDF5 file called testfile.h5 was created'
  write(*,*) 'You can check its contents e.g. using the h5dump utility'
  write(*,*) ' '

  write(*,*) 'End of TestHDF5'

end program TestHDF5
