program TestNetCDFreader
  ! ----------------------------
  !  #[ documentation
  !
  ! Test Program to load a  NetCDF file and print all its components
  ! (only the components that we use for scatterometer processing
  !  have been implemented so far)
  !
  ! Last Modified on: $Date$
  !
  !  #]
  !  #[ modules used
!  use NetCDFMod, only: nc_type_byte, nc_type_char, &
!       nc_type_short, nc_type_int, nc_type_float, nc_type_double

  use NetcdfReaderWriter, only: netcdf_data_type, read_netcdf_file, &
       delete_netcdf_data, print_dim, print_attr, print_var

  use Compiler_Features, only: getarg_genscat, iargc_genscat
  !use numerics, only: r8_, r4_, i4_, i2_
  use ErrorHandler, only: no_error
  !  #]
  !  #[ variables, types and parameters

  implicit none

  integer :: narg, error_flag, i
  character(len=256) :: filename_nc_in
  type(netcdf_data_type) :: ncdata

  !  #]
  !  #[ program
  
  ! initialisation
  filename_nc_in      = '[undefined]'

  ! command line handling
  narg = iargc_genscat()
  if (narg .ne. 1) then
    print *,'ERROR: wrong number of commandline inputs.'
    print *,''
    print *,'Usage: '
    print *,'nc_L2_to_L3 <NetCDF input file>'
    print *,' '
    stop
  endif

  call getarg_genscat(1,filename_nc_in)


  !  load data from input NetCDF file

  print *,'Read NetCDF data from file ', trim(filename_nc_in)
  print *,"=================="
  call read_netcdf_file(filename_nc_in, ncdata, error_flag)
  if (error_flag .ne. no_error) then
     print *,"ERROR: reading of netcdf file failed for file: "
     print *,"==>"//trim(filename_nc_in)
     stop 1
  end if

  ! print the results
  print *,'ndims = ', ncdata%ndims,' (dimensions)'
  print *,'nvars = ', ncdata%nvars,' (variables)'
  print *,'natts = ', ncdata%natts,' (global attributes)'
  print *,"=================="
  do i=1,ncdata%ndims
     call print_dim(ncdata%dimensions(i-1))
  end do

  print *,"=================="
  do i=1,ncdata%natts
     call print_attr(ncdata%attributes(i-1))
  end do

  print *,'=================='
  do i=1,ncdata%nvars
     call print_var(ncdata%variables(i-1))
  end do
  print *,'=================='

  ! close NetCDF input file
  print *,'closing'
  call delete_netcdf_data(ncdata, error_flag)
  if (error_flag .ne. no_error) then
     print *,"ERROR: delete_netcdf_data call failed"
     stop 1
  end if
  
  !  #]
  ! ----------------------------
end program TestNetCDFreader
