module NetcdfReaderWriter
  !  #[ modules used
  use NetCDFMod, only: undef_short, undef_int, undef_float, undef_double, &
       nc_enddef, nc_open, nc_close, &
       nc_global, nc_inq, nc_inq_attname, nc_inq_attid, nc_inq_atttype, &
       nc_get_att_text, nc_inq_attlen, nc_get_att_short, nc_get_att_int, &
       nc_get_att_float, nc_get_att_double, nc_inq_varname, &
       nc_inq_vartype, nc_inq_varndims, nc_inq_vardimid, nc_inq_varnatts, &
       nc_inq_dim, nc_get_att_int_1d, &
       nc_get_var_short_1d, nc_get_var_short_2d, nc_get_var_short_3d, &
       nc_get_var_int_1d, nc_get_var_int_2d, nc_get_var_int_3d, &
       nc_get_var_float_1d, nc_get_var_float_2d, nc_get_var_float_3d, &
       nc_get_var_double_1d, nc_get_var_double_2d, nc_get_var_double_3d, &
       nc_get_var_byte_1d, nc_get_var_byte_2d,nc_get_var_text_2d,&
       nc_type_byte, nc_type_char, nc_type_short, nc_type_int, &
       nc_type_float, nc_type_double, nc_type_nat, nc_create, &
       nc_put_att_int, nc_put_att_text, &
       nc_put_att_float, nc_put_att_double, &
       nc_put_att_short, nc_def_dim, dtype_to_text, &
       nc_def_var_short, nc_def_var_int, nc_def_var_float, nc_def_var_double, &
       nc_put_array_short_1d, nc_put_array_short_2d, nc_put_array_short_3d, &
       nc_put_array_int_1d, nc_put_array_int_2d, nc_put_array_int_3d, &
       nc_put_array_float_1d, nc_put_array_float_2d, nc_put_array_float_3d, &
       nc_put_array_double_1d, nc_put_array_double_2d, nc_put_array_double_3d, &
       get_bytesize_short, get_bytesize_int, get_bytesize_long, &
       get_bytesize_float, get_bytesize_double, &
       nc_get_array_double_2d, nc_get_array_float_2d, &
       nc_get_array_byte_2d, nc_get_array_text_2d, &
       nc_get_array_int_2d, nc_get_array_short_2d, &
       nc_get_array_double_3d, nc_get_array_float_3d, &
       nc_get_array_int_3d, nc_get_array_short_3d, nc_put_att_int_arr
  use ErrorHandler, only: no_error, error_opening_file, &
       error_closing_file, error_allocate, error_netcdf_lib, &
       error_not_yet_implemented
  use numerics, only: i1_, i2_, i4_, r4_, r8_, missing_indicator_real_r8, &
       missing_indicator_integer_i2_, missing_indicator_integer_i4_, &
       missing_real
  !  #]
  !  #[ variables and types
  implicit none

  type dim_type
     character(len=255) :: name
     integer :: length
  end type dim_type
  
  type attr_type
     character(len=255) :: name
     integer :: dtype
     integer :: len
     ! only filled for text type
     character(len=512) :: text
     ! only filled for byte type, we use i2_ length to be able to use existing reading routines
     integer(i2_) :: byte_value
     ! only filled for short type
     integer(i2_) :: short_value
     ! only filled for int type
     integer(i4_) :: int_value
     ! only filled for float type
     real(r4_) :: float_value
     ! only filled for double type
     real(r8_) :: double_value
     ! only filled for 1d int array type
     integer(i4_), dimension(:), pointer :: int_1d_data
  end type attr_type

  type var_type
     character(len=255) :: name
     integer :: dtype
     integer :: ndims
     ! we have no more than 3 dimensions in our data for now
     integer, dimension(0:2) :: dimid
     ! todo: type(dim_type), dimension(:), pointer ...
     integer :: natts
     type(attr_type), dimension(:), pointer :: attributes
     ! only filled for 1d types
     integer(i2_), dimension(:), pointer :: short_1d_data
     integer(i1_), dimension(:), pointer :: byte_1d_data
     integer(i4_), dimension(:), pointer :: int_1d_data
     real(r4_), dimension(:), pointer :: float_1d_data
     real(r8_), dimension(:), pointer :: double_1d_data
     ! only filled for 2d types
     integer(i2_), dimension(:,:), pointer :: short_2d_data 
     integer(i4_), dimension(:,:), pointer :: int_2d_data
     real(r4_), dimension(:,:), pointer :: float_2d_data
     real(r8_), dimension(:,:), pointer :: double_2d_data
     integer(i1_), dimension(:,:), pointer :: byte_2d_data
     character(len=1),dimension(:,:), pointer :: char_2d_data
     ! only filled for 3d types
     integer(i2_), dimension(:,:,:), pointer :: short_3d_data 
     integer(i4_), dimension(:,:,:), pointer :: int_3d_data
     real(r4_), dimension(:,:,:), pointer :: float_3d_data
     real(r8_), dimension(:,:,:), pointer :: double_3d_data
 end type var_type
  
  type netcdf_data_type
     integer :: unit_nc
     integer :: natts, ndims, nvars
     type(dim_type), dimension(:), pointer :: dimensions
     type(attr_type), dimension(:), pointer :: attributes
     type(var_type), dimension(:), pointer :: variables
  end type netcdf_data_type

  !  #]
contains
  !---------------------------
  !  #[ handle dim_type
  subroutine init_dim(dim)
    !  #[ init
    type(dim_type), intent(out) :: dim
    dim%name(:) = ' '
    dim%length = 0
  end subroutine init_dim
    !  #]

  ! no pointers, so no delete routine needed

  subroutine print_dim(dim)
    !  #[ print
    type(dim_type), intent(in) :: dim
    print '(4A,i11)','dimension: name = [',trim(dim%name),']',&
         ' length = ', dim%length
  end subroutine print_dim
    !  #]
  !  #]
  !  #[ handle attr_type
  subroutine init_attr(attr)
    !  #[ init
    type(attr_type), intent(out) :: attr
    attr%name(:)   = ' '
    attr%dtype     = nc_type_nat
    attr%len       = 0
    attr%text(:)   = ' '
    attr%short_value = 0
    attr%int_value = 0
    attr%float_value = 0._r4_
    attr%double_value = 0._r8_
    nullify(attr%int_1d_data)
  end subroutine init_attr
    !  #]
  subroutine delete_attr(attr)
    !  #[ delete
    type(attr_type), intent(inout) :: attr

    if (associated(attr%int_1d_data)) deallocate(attr%int_1d_data)

  end subroutine delete_attr
    !  #]
  subroutine load_attr(unit_nc, varid, attr_id, attribute, error_flag)
    !  #[ load
    integer, intent(in) :: unit_nc
    integer, intent(in) :: attr_id
    integer, intent(in) :: varid
    type(attr_type), intent(out) :: attribute
    integer, intent(out) :: error_flag

    ! local variables
    integer      :: st, attr_dtype, attr_intvalue, attr_len, alloc_status
    integer(i2_) :: attr_shortvalue, attr_bytevalue
    real(r4_)    :: attr_floatvalue
    real(r8_)    :: attr_doublevalue
    character(len=255) :: attr_name
    character(len=512) :: attr_text_value

    error_flag = no_error

    st = nc_inq_attname(unit_nc, varid, attr_id, attr_name)
    if (st .ne. 0) then
       print *,"ERROR in call to: nc_inq_attname"
       error_flag = error_netcdf_lib
       return
    end if
    attribute%name      = attr_name

    st = nc_inq_atttype(unit_nc, varid, attr_name, attr_dtype)
    if (st .ne. 0) then
       print *,"ERROR in call to: nc_inq_atttype"
       error_flag = error_netcdf_lib
       return
    end if
    attribute%dtype     = attr_dtype
    
    st = nc_inq_attlen(unit_nc, varid, attr_name, attr_len)
    if (st .ne. 0) then
       print *,"ERROR in call to: nc_inq_attlen"
       error_flag = error_netcdf_lib
       return
    end if
    attribute%len       = attr_len

    attribute%text      = ''
    attribute%int_value = -1
    
    select case(attribute%dtype)
    case(nc_type_char)
       
       ! verify length
       if (len(attr_text_value) .lt. attribute%len) then
          print *,"Cannot load attribute content for attribute: "//&
               trim(attr_name)
          print *,"attribute text length is ", attribute%len
          print *,"but available fortran string length is only: ",&
               len(attr_text_value)
          attribute%text = ' '
       else
          st = nc_get_att_text(unit_nc, varid, attr_name, attr_text_value)
          if (st .ne. 0) then
             print *,"ERROR in call to: nc_get_att_text"
             error_flag = error_netcdf_lib
             return
          end if
          attribute%text = attr_text_value(1:attribute%len)
       end if
    case(nc_type_short)
       ! verify length
       if (attribute%len .ne. 1) then
          print *,"Cannot load attribute content for attribute: "//&
               trim(attr_name)
          print *,"loading an array of short integers for attributes"
          print *,"is not yet implemented."
       else
          st = nc_get_att_short(unit_nc, varid, attr_name, attr_shortvalue)
          if (st .ne. 0) then
             print *,"ERROR in call to: nc_get_att_short"
             error_flag = error_netcdf_lib
             return
          end if
          attribute%short_value = attr_shortvalue
       end if
    case(nc_type_int)

       ! verify length
       if (attribute%len .eq. 1) then
          st = nc_get_att_int(unit_nc, varid, attr_name, attr_intvalue)
          if (st .ne. 0) then
             print *,"ERROR in call to: nc_get_att_int"
             error_flag = error_netcdf_lib
             return
          end if
          attribute%int_value = attr_intvalue
       else
          allocate(attribute%int_1d_data(0:attribute%len-1), stat=alloc_status)
          if (alloc_status .ne. 0) then
             print *,'ERROR: allocation problem in load_attr'
             error_flag = error_allocate
             return
          end if
          attribute%int_1d_data(:) = 0

          st = nc_get_att_int_1d(unit_nc, varid, attribute%len, attr_name, &
                                 attribute%int_1d_data)
          if (st .ne. 0) then
             print *,"ERROR in call to: nc_get_att_int_1d"
             error_flag = error_netcdf_lib
             return
          end if

       end if
    case(nc_type_double)
       ! verify length
       if (attribute%len .ne. 1) then
          print *,"ERROR: loading an array of doubles for attributes"
          print *,"is not yet implemented."
          error_flag = error_netcdf_lib
          return
       end if

       st = nc_get_att_double(unit_nc, varid, attr_name, attr_doublevalue)
       if (st .ne. 0) then
          print *,"ERROR in call to: nc_get_att_int"
          error_flag = error_netcdf_lib
          return
       end if
       attribute%double_value = attr_doublevalue
    case(nc_type_float)
       ! verify length
       if (attribute%len .ne. 1) then
          print *,"ERROR: loading an array of floats for attributes"
          print *,"is not yet implemented."
          error_flag = error_netcdf_lib
          return
       end if

       st = nc_get_att_float(unit_nc, varid, attr_name, attr_floatvalue)
       if (st .ne. 0) then
          print *,"ERROR in call to: nc_get_att_int"
          error_flag = error_netcdf_lib
          return
       end if
       attribute%float_value = attr_floatvalue
    case(nc_type_byte)
       ! verify length
       if (attribute%len .ne. 1) then
          print *,"ERROR: loading an array of bytes for attributes"
          print *,"is not yet implemented."
          error_flag = error_netcdf_lib
          return
       end if

       st = nc_get_att_short(unit_nc, varid, attr_name, attr_bytevalue)
       if (st .ne. 0) then
          print *,"ERROR in call to: nc_get_att_short"
          error_flag = error_netcdf_lib
          return
       end if
       attribute%byte_value = attr_bytevalue
    case default
       print *,'attr_name = ['//trim(attr_name)//']'
       print *,'handling attr. type: ', attr_dtype,&
            ' is not yet implemented.'
    end select

  end subroutine load_attr
    !  #]
  subroutine write_attr(unit_nc, varid, attribute, error_flag)
    !  #[ write to netcdf file
    integer, intent(in) :: unit_nc
    integer, intent(in) :: varid
    type(attr_type), intent(in) :: attribute
    integer, intent(out) :: error_flag

    ! local variables
    integer :: st
    
    !print *,"debug: writing attribute: "//trim(attribute%name)
    !print *,'debug: varid = ',varid
    
    select case(attribute%dtype)
    case(nc_type_char)
       st = nc_put_att_text(unit_nc, varid,&
                            trim(attribute%name), trim(attribute%text))
       if (st .ne. 0) then
          print *,"ERROR in call to: nc_put_att_text"
          error_flag = error_netcdf_lib
          return
       end if
    case(nc_type_short)
       st = nc_put_att_short(unit_nc, varid, &
                             trim(attribute%name), attribute%short_value)
       if (st .ne. 0) then
          print *,"ERROR in call to nc_put_att_short"
          error_flag = error_netcdf_lib
          return
       end if
    case(nc_type_int)
       ! verify length
       if (attribute%len .eq. 1) then
          st = nc_put_att_int(unit_nc, varid,&
                              trim(attribute%name), attribute%int_value)
          if (st .ne. 0) then
             print *,"ERROR in call to: nc_put_att_int"
             error_flag = error_netcdf_lib
             return
          end if
       else ! this is an int array, not a scalar
          st = nc_put_att_int_arr(unit_nc, varid, trim(attribute%name), &
                                  attribute%len, attribute%int_1d_data)
          if (st .ne. 0) then
             print *,"ERROR in call to: nc_put_att_int_arr"
             error_flag = error_netcdf_lib
             return
          end if
       end if
    case(nc_type_float)
       st = nc_put_att_float(unit_nc, varid, &
                             trim(attribute%name), attribute%float_value)
       if (st .ne. 0) then
          print *,"ERROR in call to nc_put_att_float"
          error_flag = error_netcdf_lib
          return
       end if
    case(nc_type_double)
       st = nc_put_att_double(unit_nc, varid, &
                              trim(attribute%name), attribute%double_value)
       if (st .ne. 0) then
          print *,"ERROR in call to nc_put_att_double"
          error_flag = error_netcdf_lib
          return
       end if
    case default
       print *,"sorry, writing attributes with type "//&
            trim(dtype_to_text(attribute%dtype))//&
            " is not yet implemented"
       error_flag = error_netcdf_lib
       return
    end select
    
  end subroutine write_attr
    !  #]
  subroutine print_attr(attr, prefix)
    !  #[ print
    type(attr_type), intent(in) :: attr
    character(len=*), optional :: prefix

    ! local variables
    character(len=256) :: prefix_to_use

    prefix_to_use(:) = ' '
    if (present(prefix)) prefix_to_use = prefix

    select case(attr%dtype)
    case(nc_type_char)
       print '(7A)',trim(prefix_to_use),'attribute name: [',&
            trim(attr%name),']',' value: [',trim(attr%text),']'
    case(nc_type_short)
       print '(5A,i11,A)',trim(prefix_to_use),'attribute name: [',&
            trim(attr%name),']', ' value: ', attr%short_value, ' // short'
    case(nc_type_int)
       if (associated(attr%int_1d_data)) then
          ! we have an array of integers
          print '(4A)',trim(prefix_to_use),'attribute name: [',&
               trim(attr%name),']'
          print *,' values: ', attr%int_1d_data, ' // int array 1d'
       else
          ! we have just a single integer
          print '(5A,i11,A)',trim(prefix_to_use),'attribute name: [',&
               trim(attr%name),']',' value: ', attr%int_value, ' // int'
       end if
    case(nc_type_float)
       print '(5A,e11.3,A)',trim(prefix_to_use),'attribute name: [',&
            trim(attr%name),']',' value: ', attr%float_value, ' // float'
    case(nc_type_double)
       print '(5A,e11.3,A)',trim(prefix_to_use),'attribute name: [',&
            trim(attr%name),']',' value: ', attr%double_value, ' // double'
    case default
       print *,trim(prefix_to_use),'attribute name: ['//trim(attr%name)//']'
       print *,trim(prefix_to_use),'handling attr. type: ', attr%dtype, &
            ' is not yet implemented.'
    end select
  end subroutine print_attr
    !  #]
  subroutine copy_attr_content(source_attr, dest_attr, error_flag)
    !  #[ move content of one attr to another
    type(attr_type), intent(in)    :: source_attr
    type(attr_type), intent(out)   :: dest_attr
    integer,         intent(out)   :: error_flag
    
    ! local variable
    integer :: alloc_status

    error_flag = no_error
    
    dest_attr%name         = source_attr%name
    dest_attr%dtype        = source_attr%dtype
    dest_attr%text         = source_attr%text
    dest_attr%len          = source_attr%len
    dest_attr%short_value  = source_attr%short_value
    dest_attr%int_value    = source_attr%int_value
    dest_attr%float_value  = source_attr%float_value
    dest_attr%double_value = source_attr%double_value
    
    nullify(dest_attr%int_1d_data)
    if (associated(source_attr%int_1d_data)) then
       allocate(dest_attr%int_1d_data(source_attr%len), stat=alloc_status)
       if (alloc_status .ne. 0) then
          print *,'ERROR: allocation problem in copy_attr_content'
          error_flag = error_allocate
          return
       end if
       dest_attr%int_1d_data(:) = source_attr%int_1d_data(:)
    end if

  end subroutine copy_attr_content
    !  #]
  subroutine xmove_attr_content(source_attr, dest_attr)
    !  #[ move content of one attr to another
    type(attr_type), intent(inout) :: source_attr
    type(attr_type), intent(out)   :: dest_attr
    
    dest_attr%name         = source_attr%name
    dest_attr%dtype        = source_attr%dtype
    dest_attr%text         = source_attr%text
    dest_attr%len          = source_attr%len
    dest_attr%short_value  = source_attr%short_value
    dest_attr%int_value    = source_attr%int_value
    dest_attr%float_value  = source_attr%float_value
    dest_attr%double_value = source_attr%double_value
    
    dest_attr%int_1d_data  => source_attr%int_1d_data
    nullify(source_attr%int_1d_data)

  end subroutine xmove_attr_content
    !  #]
  !  #]
  !  #[ handle var_type
  subroutine init_var(var)
    !  #[ init
    type(var_type), intent(out) :: var

    var%name(:) = ' '
    var%dtype   = nc_type_nat
    var%ndims   = 0
    var%dimid   = (/0,0,0/)
    var%natts   = 0

    nullify(var%attributes)

    nullify(var%short_1d_data)
    nullify(var%short_2d_data)
    nullify(var%short_3d_data)

    nullify(var%int_1d_data)
    nullify(var%int_2d_data)
    nullify(var%int_3d_data)

    nullify(var%float_1d_data)
    nullify(var%float_2d_data)
    nullify(var%float_3d_data)

    nullify(var%double_1d_data)
    nullify(var%double_2d_data)
    nullify(var%double_3d_data)

  end subroutine init_var
    !  #]
  subroutine delete_var(var)
    !  #[ delete
    type(var_type), intent(inout) :: var
    
    ! local variable
    integer :: i

    if (associated(var%attributes)) then
       do i=1, var%natts
          call delete_attr(var%attributes(i-1))
       end do
       deallocate(var%attributes)
    end if

    if (associated(var%short_1d_data)) deallocate(var%short_1d_data)
    if (associated(var%short_2d_data)) deallocate(var%short_2d_data)
    if (associated(var%short_3d_data)) deallocate(var%short_3d_data)

    if (associated(var%int_1d_data)) deallocate(var%int_1d_data)
    if (associated(var%int_2d_data)) deallocate(var%int_2d_data)
    if (associated(var%int_3d_data)) deallocate(var%int_3d_data)

    if (associated(var%float_1d_data)) deallocate(var%float_1d_data)
    if (associated(var%float_2d_data)) deallocate(var%float_2d_data)
    if (associated(var%float_3d_data)) deallocate(var%float_3d_data)

    if (associated(var%double_1d_data)) deallocate(var%double_1d_data)
    if (associated(var%double_2d_data)) deallocate(var%double_2d_data)
    if (associated(var%double_3d_data)) deallocate(var%double_3d_data)

  end subroutine delete_var
    !  #]
  subroutine print_var(var)
    !  #[ print
    type(var_type), intent(in) :: var

    ! local variable
    integer :: i, n

    print '(3A)','variable name: [',trim(var%name),']'

    select case(var%ndims)
    case(1)
       print '(A,A,A,i2,A,i2,A,i2)','type ',dtype_to_text(var%dtype), &
            ' ndims ',var%ndims, &
            ' dimid0 ', var%dimid(0), ' natts = ', var%natts
    case(2)
       print '(A,A,A,i2,A,2i2,A,i2)','type ',dtype_to_text(var%dtype), &
            ' ndims ',var%ndims, &
            ' dimid0/1 ', var%dimid(0), var%dimid(1), ' natts = ', var%natts
    case(3)
       print '(A,A,A,i2,A,3i2,A,i2)','type ',dtype_to_text(var%dtype), &
            ' ndims ',var%ndims, &
            ' dimid0/1 ', var%dimid(0), var%dimid(1), var%dimid(2), &
            ' natts = ', var%natts
    case default
       print *,'handling 4 or more dims is not yet implemented' 
       print '(A,A,A,i2,A,i2)','type ',dtype_to_text(var%dtype), &
            ' ndims ',var%ndims, &
            ' natts = ', var%natts
    end select
    
    !print *,"TEST: var%name ", trim(var%name)
    !print *,"TEST: var%dtype ", var%dtype
    !print *,"TEST: var%ndims ", var%ndims
    !print *,"TEST: var%dimid ", var%dimid
    !print *,"TEST: var%natts ", var%natts
    !print *,"TEST: var%attributes ", associated(var%attributes)
    !print *,"TEST: var%short_1d_data ", associated(var%short_1d_data)
    !print *,"TEST: var%int_1d_data ", associated(var%int_1d_data)
    !print *,"TEST: var%float_1d_data ", associated(var%float_1d_data)
    !print *,"TEST: var%double_1d_data ", associated(var%double_1d_data)
    !print *,"TEST: var%short_2d_data ", associated(var%short_2d_data)
    !print *,"TEST: var%int_2d_data ", associated(var%int_2d_data)
    !print *,"TEST: var%float_2d_data ", associated(var%float_2d_data)
    !print *,"TEST: var%double_2d_data ", associated(var%double_2d_data)
    !print *,"TEST: var%short_3d_data ", associated(var%short_3d_data)
    !print *,"TEST: var%int_3d_data ", associated(var%int_3d_data)
    !print *,"TEST: var%float_3d_data ", associated(var%float_3d_data)
    !print *,"TEST: var%double_3d_data ", associated(var%double_3d_data)

    print *,"=================="

    if (var%ndims .eq. 1) then
       select case(var%dtype)
       case(nc_type_short)
          n = size(var%short_1d_data)
          if (n .gt. 5) n=5
          print '(A,i1,A,5i7)','==>var%short_1d_data(1:',n,') = ', &
               var%short_1d_data(1:n)
       case(nc_type_int)
          n = size(var%int_1d_data)
          if (n .gt. 5) n=5
          print '(A,i1,A,5i12)','==>var%int_1d_data(1:',n,') = ', &
               var%int_1d_data(1:n)
       case(nc_type_float)
          n = size(var%float_1d_data)
          if (n .gt. 5) n=5
          print '(A,i1,A,5e12.3)','==>var%float_1d_data(1:',n,') = ', &
               var%float_1d_data(1:n)
       case(nc_type_double)
          n = size(var%double_1d_data)
          if (n .gt. 5) n=5
          print '(A,i1,A,5d12.3)','==>var%double_1d_data(1:',n,') = ', &
               var%double_1d_data(1:n)
       case default
          print *,'==>printing 1D arrays is not yet implemented '//&
               'for variable type ',var%dtype
       end select
    end if
    
    if (var%ndims .eq. 2) then
       select case(var%dtype)
       case(nc_type_short)
          print '(A,5i7)','==>var%short_2d_data(1:5,1) = ', &
               var%short_2d_data(1:5,1)
       case(nc_type_int)
          print '(A,5i10)','==>var%int_2d_data(1:5,1) = ', &
               var%int_2d_data(1:5,1)
       case(nc_type_float)
          print '(A,5i10)','==>var%float_2d_data(1:5,1) = ', &
               var%float_2d_data(1:5,1)
       case(nc_type_double)
          print '(A,5i10)','==>var%double_2d_data(1:5,1) = ', &
               var%double_2d_data(1:5,1)
       case default
         print *,'==>printing 2D arrays is not yet implemented '//&
               'for variable type ',var%dtype
       end select
    end if

    if (var%ndims .eq. 3) then
       select case(var%dtype)
       case(nc_type_short)
          print '(A,5i7)','==>var%short_3d_data(1,1:5,1) = ', &
               var%short_3d_data(1,1:5,1)
       case(nc_type_int)
          print '(A,5i10)','==>var%int_3d_data(1,1:5,1) = ', &
               var%int_3d_data(1,1:5,1)
       case(nc_type_float)
          print '(A,5i10)','==>var%float_3d_data(1,1:5,1) = ', &
               var%float_3d_data(1,1:5,1)
       case(nc_type_double)
          print '(A,5i10)','==>var%double_3d_data(1,1:5,1) = ', &
               var%double_3d_data(1,1:5,1)
       case default
         print *,'==>printing 2D arrays is not yet implemented '//&
               'for variable type ',var%dtype
       end select
    end if
    
    do i=1,var%natts
       call print_attr(var%attributes(i-1),prefix="==>")
    end do

  end subroutine print_var
    !  #]
  subroutine get_2d_var_values_as_real8(var, copied_values, error_flag)
    !  #[ convert to a real8 array,
    !     taking scaling, offset, and missing into account
    type(var_type), intent(in) :: var
    real(r8_), dimension(:,:), pointer :: copied_values
    integer, intent(out) :: error_flag

    ! local variables
    integer :: i, j, size1, size2, status
    integer(i2_) :: missing_short_data_value
    integer(i4_) :: missing_int_data_value
    real(r8_) :: scaling_data_value
    real(r8_) :: offset_data_value
    
    ! init
    error_flag = no_error
    missing_short_data_value = 0
    missing_int_data_value = 0
    nullify(copied_values)
    
    ! get the scaling and offset values
    scaling_data_value = get_double_attribute(var, 'scale_factor')
    offset_data_value  = get_double_attribute(var, 'add_offset')

    ! allow this one to be missing, assume no scaling if this is the case
    if (missing_real(scaling_data_value)) then
       scaling_data_value = 1._r8_
    end if
    
    ! allow this one to be missing, assume no offset if this is the case
    if (missing_real(offset_data_value)) then
       offset_data_value = 0._r8_
    end if
    
    select case(var%dtype)
    case(nc_type_short)
       ! get the missing value
       missing_short_data_value = get_short_attribute(var, 'missing_value')
       ! allocate the real8 array
       size1 = size(var%short_2d_data, 1)
       size2 = size(var%short_2d_data, 2)
       allocate(copied_values(size1, size2),stat=status)
       if (status .ne. 0) then
          error_flag = error_allocate
          return
       end if
       ! copy the data array
       do i=1,size1
          do j=1,size2
             if (var%short_2d_data(i,j) .eq. missing_short_data_value) then
                copied_values(i,j) = missing_indicator_real_r8
             else
                copied_values(i,j) = offset_data_value + &
                     real(var%short_2d_data(i,j), r8_)*scaling_data_value
             end if
          end do
       end do
   case(nc_type_int)
      ! get the missing value
      missing_int_data_value = get_int_attribute(var, 'missing_value')
      ! allocate the real8 array
       size1 = size(var%int_2d_data, 1)
       size2 = size(var%int_2d_data, 2)
       allocate(copied_values(size1, size2),stat=status)
       if (status .ne. 0) then
          error_flag = error_allocate
          return
       end if
       ! copy the data array
       do i=1,size1
          do j=1,size2
             if (var%int_2d_data(i,j) .eq. missing_int_data_value) then
                copied_values(i,j) = missing_indicator_real_r8
             else
                copied_values(i,j) = offset_data_value + &
                     real(var%int_2d_data(i,j), r8_)*scaling_data_value
             end if
          end do
       end do
    case default
       print *,"ERROR in get_var_values_as_real8: "//&
            "handling of this datatype is not yet implemented: ", var%dtype
       stop 1
    end select
    
  end subroutine get_2d_var_values_as_real8
    !  #]
  subroutine move_var_content(source_var, dest_var)
    !  #[ move content of one var to another
    type(var_type), intent(inout) :: source_var
    type(var_type), intent(out)   :: dest_var

    dest_var%name  = source_var%name
    dest_var%dtype = source_var%dtype
    dest_var%ndims = source_var%ndims
    dest_var%dimid = source_var%dimid
    dest_var%natts = source_var%natts

    dest_var%attributes     => source_var%attributes
    nullify(source_var%attributes)

    dest_var%short_1d_data  => source_var%short_1d_data
    dest_var%int_1d_data    => source_var%int_1d_data
    dest_var%float_1d_data  => source_var%float_1d_data
    dest_var%double_1d_data => source_var%double_1d_data
    nullify(source_var%short_1d_data, &
            source_var%int_1d_data, &
            source_var%float_1d_data, &
            source_var%double_1d_data)

    dest_var%short_2d_data  => source_var%short_2d_data
    dest_var%int_2d_data    => source_var%int_2d_data
    dest_var%float_2d_data  => source_var%float_2d_data
    dest_var%double_2d_data => source_var%double_2d_data
    nullify(source_var%short_2d_data, &
            source_var%int_2d_data, &
            source_var%float_2d_data, &
            source_var%double_2d_data)

    dest_var%short_3d_data  => source_var%short_3d_data
    dest_var%int_3d_data    => source_var%int_3d_data
    dest_var%float_3d_data  => source_var%float_3d_data
    dest_var%double_3d_data => source_var%double_3d_data
    nullify(source_var%short_3d_data, &
            source_var%int_3d_data, &
            source_var%float_3d_data, &
            source_var%double_3d_data)

  end subroutine move_var_content
    !  #]
  function has_attribute(var, attr_name) result(attr_present)
    !  #[ has attr
    type(var_type),   intent(in) :: var          ! input
    character(len=*), intent(in) :: attr_name    ! input
    logical                      :: attr_present ! result
    
    ! local variable
    integer :: i

    attr_present = .false.
    attr_loop: do i=1,var%natts
       if (trim(var%attributes(i-1)%name) .eq. trim(attr_name)) then
          attr_present = .true.
          exit attr_loop
       end if
    end do attr_loop

  end function has_attribute
    !  #]
  function get_text_attribute(var, attr_name) result(text_attr)
    !  #[ get text value
    type(var_type),   intent(in) :: var          ! input
    character(len=*), intent(in) :: attr_name    ! input
    character(len=512)           :: text_attr    ! result
    
    ! local variable
    integer :: i

    text_attr(:) = ' '
    attr_loop: do i=1,var%natts
       if (trim(var%attributes(i-1)%name) .eq. trim(attr_name)) then
          text_attr = var%attributes(i-1)%text
          exit attr_loop
       end if
    end do attr_loop

  end function get_text_attribute
    !  #]
  function get_short_attribute(var, attr_name) result(short_attr)
    !  #[ get short value
    type(var_type),   intent(in) :: var        ! input
    character(len=*), intent(in) :: attr_name  ! input
    integer(i2_)                 :: short_attr ! result
    
    ! local variable
    integer :: i

    short_attr = missing_indicator_integer_i2_
    attr_loop: do i=1,var%natts
       if (trim(var%attributes(i-1)%name) .eq. trim(attr_name)) then
          short_attr = var%attributes(i-1)%short_value
          exit attr_loop
       end if
    end do attr_loop

  end function get_short_attribute
    !  #]
  function get_int_attribute(var, attr_name) result(int_attr)
    !  #[ get int value
    type(var_type),   intent(in) :: var       ! input
    character(len=*), intent(in) :: attr_name ! input
    integer(i4_)                 :: int_attr  ! result
    
    ! local variable
    integer :: i

    int_attr = missing_indicator_integer_i4_
    attr_loop: do i=1,var%natts
       if (trim(var%attributes(i-1)%name) .eq. trim(attr_name)) then
          int_attr = var%attributes(i-1)%int_value
          exit attr_loop
       end if
    end do attr_loop

  end function get_int_attribute
    !  #]
  function get_double_attribute(var, attr_name) result(double_attr)
    !  #[ get double value
    type(var_type),   intent(in) :: var          ! input
    character(len=*), intent(in) :: attr_name    ! input
    real(r8_)                    :: double_attr  ! result
    
    ! local variable
    integer :: i

    double_attr = missing_indicator_real_r8
    attr_loop: do i=1,var%natts
       if (trim(var%attributes(i-1)%name) .eq. trim(attr_name)) then
          double_attr = var%attributes(i-1)%double_value
          exit attr_loop
       end if
    end do attr_loop

  end function get_double_attribute
    !  #]
  subroutine add_attribute(variable, new_attr, error_flag)
    !  #[ add a new attribute to a variable
    type(var_type),  intent(inout) :: variable
    type(attr_type), intent(in)    :: new_attr
    integer,         intent(out)   :: error_flag

    ! local variables
    integer :: i, alloc_status, prev_num_atts, new_num_atts
    type(attr_type), dimension(:), pointer :: prev_attributes
    type(attr_type), dimension(:), pointer :: new_attributes
    
    error_flag = no_error

    prev_attributes => variable%attributes ! could be nullified
    prev_num_atts = variable%natts         ! could be 0
    new_num_atts = prev_num_atts+1         ! should be 1 or more

    allocate(new_attributes(0:new_num_atts-1), stat=alloc_status)
    if (alloc_status .ne. 0) then
       print *,'ERROR: allocation problem in add_attribute'
       error_flag = error_allocate
       return
    end if

    if (new_num_atts .gt. 1) then
       ! an attributes array already exists, so copy the existing data
       do i=1,prev_num_atts
          call copy_attr_content(prev_attributes(i-1), new_attributes(i-1),&
                                 error_flag)
          if (error_flag .ne. no_error) return
       end do
    end if
    
    ! finally add the new attribute
    variable%natts = new_num_atts

    call copy_attr_content(new_attr, new_attributes(new_num_atts-1), &
                           error_flag)
    if (error_flag .ne. no_error) return

    !do i=1,prev_num_atts
    !   call delete_attr(prev_attributes(i-1))
    !end do

! deze call zorgt voor de memory corruptie !
    !if (associated(prev_attributes)) deallocate(prev_attributes)

    variable%attributes => new_attributes

  end subroutine add_attribute
    !  #]
  subroutine add_short_attr(variable, attr_name, attr_short_value, error_flag)
    !  #[ add a short valued attribute
    type(var_type),   intent(inout) :: variable
    character(len=*), intent(in)    :: attr_name
    integer(i2_),     intent(in)    :: attr_short_value
    integer,          intent(out)   :: error_flag
    
    ! local variables
    type(attr_type) :: new_attr

    error_flag = no_error
    call init_attr(new_attr)
    new_attr%name        = trim(attr_name)
    new_attr%dtype       = nc_type_short
    new_attr%short_value = attr_short_value
    new_attr%len         = 1
    call add_attribute(variable, new_attr, error_flag)
    if (error_flag .ne. no_error) return

  end subroutine add_short_attr
    !  #]
  subroutine add_int_attr(variable, attr_name, attr_int_value, error_flag)
    !  #[ add an integer valued attribute
    type(var_type),   intent(inout) :: variable
    character(len=*), intent(in)    :: attr_name
    integer(i4_),     intent(in)    :: attr_int_value
    integer,          intent(out)   :: error_flag
    
    ! local variables
    type(attr_type) :: new_attr

    error_flag = no_error
    call init_attr(new_attr)
    new_attr%name      = trim(attr_name)
    new_attr%dtype     = nc_type_int
    new_attr%int_value = attr_int_value
    new_attr%len       = 1
    call add_attribute(variable, new_attr, error_flag)
    if (error_flag .ne. no_error) return

  end subroutine add_int_attr
    !  #]
  subroutine add_int_arr_attr(variable, attr_name, attr_int_arr_value, &
                              error_flag)
    !  #[ add an integer valued attribute
    type(var_type),        intent(inout) :: variable
    character(len=*),      intent(in)    :: attr_name
    integer(i4_), dimension(:), intent(in)    :: attr_int_arr_value
    integer,               intent(out)   :: error_flag
    
    ! local variables
    type(attr_type) :: new_attr

    error_flag = no_error
    call init_attr(new_attr)
    new_attr%name        = trim(attr_name)
    new_attr%dtype       = nc_type_int
    new_attr%int_1d_data = attr_int_arr_value
    new_attr%len         = size(attr_int_arr_value)
    call add_attribute(variable, new_attr, error_flag)
    if (error_flag .ne. no_error) return

  end subroutine add_int_arr_attr
    !  #]
  subroutine add_float_attr(variable, attr_name, attr_float_value, error_flag)
    !  #[ add a float valued attribute
    type(var_type),   intent(inout) :: variable
    character(len=*), intent(in)    :: attr_name
    real(r4_),        intent(in)    :: attr_float_value
    integer,          intent(out)   :: error_flag
    
    ! local variables
    type(attr_type) :: new_attr

    error_flag = no_error
    call init_attr(new_attr)
    new_attr%name        = trim(attr_name)
    new_attr%dtype       = nc_type_float
    new_attr%float_value = attr_float_value
    new_attr%len         = 1
    call add_attribute(variable, new_attr, error_flag)
    if (error_flag .ne. no_error) return

  end subroutine add_float_attr
    !  #]
  subroutine add_double_attr(variable, attr_name, attr_double_value, error_flag)
    !  #[ add an integer valued attribute
    type(var_type),   intent(inout) :: variable
    character(len=*), intent(in)    :: attr_name
    real(r8_),        intent(in)    :: attr_double_value
    integer,          intent(out)   :: error_flag
    
    ! local variables
    type(attr_type) :: new_attr

    error_flag = no_error
    call init_attr(new_attr)
    new_attr%name         = trim(attr_name)
    new_attr%dtype        = nc_type_double
    new_attr%double_value = attr_double_value
    new_attr%len          = 1
    call add_attribute(variable, new_attr, error_flag)
    if (error_flag .ne. no_error) return

  end subroutine add_double_attr
    !  #]
  subroutine add_text_attr(variable, attr_name, attr_text_value, error_flag)
    !  #[ add a text valued attribute
    type(var_type),   intent(inout) :: variable
    character(len=*), intent(in)    :: attr_name
    character(len=*), intent(in)    :: attr_text_value
    integer,          intent(out)   :: error_flag

    ! local variables
    type(attr_type) :: new_attr

    error_flag = no_error
    call init_attr(new_attr)
    new_attr%name  = trim(attr_name)
    new_attr%dtype = nc_type_char
    new_attr%text  = trim(attr_text_value)
    new_attr%len   = 1
    call add_attribute(variable, new_attr, error_flag)
    if (error_flag .ne. no_error) return

  end subroutine add_text_attr
    !  #]
  subroutine add_attr_to_named_var(ncdata, var_name, new_attr, error_flag)
    !  #[ add a new attribute to a var with given name
    type(netcdf_data_type), intent(inout) :: ncdata
    character(len=*),       intent(in)    :: var_name
    type(attr_type),        intent(in)    :: new_attr
    integer,                intent(out)   :: error_flag

    ! local variable
    integer :: varid
    
    varid = get_varid_by_name(ncdata, var_name)
    call add_attribute(ncdata%variables(varid), new_attr, error_flag)
    
  end subroutine add_attr_to_named_var
    !  #]
  subroutine add_text_attr_to_named_var(ncdata, var_name, attr_name, &
                                        attr_text_value, error_flag)
    !  #[ add a text valued attribute
    type(netcdf_data_type), intent(inout) :: ncdata
    character(len=*),       intent(in)    :: var_name
    character(len=*),       intent(in)    :: attr_name
    character(len=*),       intent(in)    :: attr_text_value
    integer,                intent(out)   :: error_flag

    ! local variables
    type(attr_type) :: new_attr

    error_flag = no_error
    call init_attr(new_attr)
    new_attr%name  = trim(attr_name)
    new_attr%dtype = nc_type_char
    new_attr%text  = trim(attr_text_value)
    new_attr%len   = 1
    call add_attr_to_named_var(ncdata, var_name, new_attr, error_flag)
    if (error_flag .ne. no_error) return

  end subroutine add_text_attr_to_named_var
    !  #]
  subroutine add_int_arr_attr_to_named_var(ncdata, var_name, attr_name, &
                              attr_int_arr_value, error_flag)
    !  #[ add an integer valued attribute
    type(netcdf_data_type), intent(inout) :: ncdata
    character(len=*),       intent(in)    :: var_name
    character(len=*),       intent(in)    :: attr_name
    integer, dimension(:),  intent(in)    :: attr_int_arr_value
    integer,                intent(out)   :: error_flag
    
    ! local variables
    type(attr_type) :: new_attr
    integer :: n, status
    
    error_flag = no_error

    call init_attr(new_attr)

    n = size(attr_int_arr_value)
    allocate(new_attr%int_1d_data(n), stat=status)
    if (status .ne. 0) then
       error_flag = error_allocate
       return
    end if

    new_attr%name        = trim(attr_name)
    new_attr%dtype       = nc_type_int
    new_attr%int_1d_data(:) = attr_int_arr_value(:)
    new_attr%len         = n
    call add_attr_to_named_var(ncdata, var_name, new_attr, error_flag)
    if (error_flag .ne. no_error) return

  end subroutine add_int_arr_attr_to_named_var
    !  #]

  !  #]
  !  #[ handle netcdf_data_type
  subroutine init_netcdf_data(netcdf_data)
    !  #[ init
    type(netcdf_data_type), intent(out) :: netcdf_data
    netcdf_data%natts   = 0
    netcdf_data%ndims   = 0
    netcdf_data%nvars   = 0
    nullify(netcdf_data%dimensions)
    nullify(netcdf_data%attributes)
    nullify(netcdf_data%variables)
  end subroutine init_netcdf_data
    !  #]
  subroutine delete_netcdf_data(netcdf_data, error_flag)
    !  #[ delete
    type(netcdf_data_type), intent(inout) :: netcdf_data
    integer, intent(out) :: error_flag

    ! local variables
    integer :: i

    ! init
    error_flag = no_error
    
    ! clean the allocated memory blocks
    if (associated(netcdf_data%variables)) then
       do i=0,size(netcdf_data%variables)-1
          call delete_var(netcdf_data%variables(i))
       end do
    end if

    if (associated(netcdf_data%dimensions)) deallocate(netcdf_data%dimensions)
    if (associated(netcdf_data%attributes)) deallocate(netcdf_data%attributes)
    if (associated(netcdf_data%variables)) deallocate(netcdf_data%variables)
  end subroutine delete_netcdf_data
    !  #]
  subroutine load_all_netcdf_items(unit_nc, netcdf_data, error_flag)
    !  #[ load all dims, vars and attrs, from the already opened file
    integer,                intent(in)  :: unit_nc
    type(netcdf_data_type), intent(out) :: netcdf_data
    integer,                intent(out) :: error_flag

    ! local variables
    integer :: i, j, st, unlimdimid, alloc_status, i_copy
    integer :: dim_length
    integer :: size0, size1, size2
    integer :: var_dimid0, var_dimid1, var_dimid2
    integer :: var_dtype, var_natts, var_ndims
    character(len=255) :: dim_name, var_name
    logical, parameter :: debug = .false. ! .true.

    ! init
    error_flag = no_error
    call init_netcdf_data(netcdf_data)

    ! get some netcdf file properties
    st = nc_inq(unit_nc, netcdf_data%ndims, netcdf_data%nvars, &
                netcdf_data%natts, unlimdimid) 
    if (st .ne. 0) then
       print *,"ERROR in call to: nc_inq"
       error_flag = error_netcdf_lib
       return
    end if
    
    if (debug) then
       print *,'netcdf_data%ndims = ', netcdf_data%ndims,' (dimensions)'
       print *,'netcdf_data%nvars = ', netcdf_data%nvars,' (variables)'
       print *,'netcdf_data%natts = ', netcdf_data%natts,' (global attributes)'
       print *,'nunlimdimid = ', unlimdimid,' (not used by us)'
    end if
    
    ! allocate
    allocate(netcdf_data%dimensions(0:netcdf_data%ndims-1), &
             netcdf_data%attributes(0:netcdf_data%natts-1), &
             netcdf_data%variables(0:netcdf_data%nvars-1), stat=alloc_status)
    if (alloc_status .ne. 0) then
       print *,'ERROR: allocation problem in read_netcdf_file'
       error_flag = error_allocate
       return
    end if
    
    ! load dimensions
    do i=1,netcdf_data%ndims
       if (debug) then
          print *,"loading dimension ",i
       end if

       call init_dim(netcdf_data%dimensions(i-1))
       
       ! BEWARE: the dimid variable seems to be changed inside the c-code
       ! during the call to nc_inq_dim, so we need to use a copy to
       ! make sure the fortran code does not get confused (using i directly
       ! gives ugly core dumps without this workaround)

       i_copy = i-1
       st = nc_inq_dim(unit_nc, i_copy, dim_name, dim_length)
       if (st .ne. 0) then
          print *,"ERROR in call to: nc_inq_dim"
          error_flag = error_netcdf_lib
          return
       end if
       netcdf_data%dimensions(i-1)%name   = dim_name
       netcdf_data%dimensions(i-1)%length = dim_length
    end do
    
    ! load global attributes
    do i=1,netcdf_data%natts
       if (debug) then
          print *,"loading attribute ",i
       end if
       call init_attr(netcdf_data%attributes(i-1))
       call load_attr(unit_nc, nc_global, i-1, &
                      netcdf_data%attributes(i-1),error_flag)
       if (error_flag .ne. no_error) return
    end do
    
    ! load variables
    varsloop: do i=1,netcdf_data%nvars
       if (debug) then
          print *,"loading variable ",i
       end if
       call init_var(netcdf_data%variables(i-1))
       
       st = nc_inq_varname(unit_nc, i-1, var_name)
       if (st .ne. 0) then
          print *,"ERROR in call to: nc_inq_varname"
          error_flag = error_netcdf_lib
          return
       end if
       
       st = nc_inq_vartype(unit_nc, i-1, var_dtype)
       if (st .ne. 0) then
          print *,"ERROR in call to: nc_inq_vartype"
          error_flag = error_netcdf_lib
          return
       end if
       
       st = nc_inq_varndims(unit_nc, i-1, var_ndims)
       if (st .ne. 0) then
          print *,"ERROR in call to: nc_inq_varndims"
          error_flag = error_netcdf_lib
          return
       end if
       
       st = nc_inq_vardimid(unit_nc, i-1, var_dimid0, var_dimid1, var_dimid2)
       if (st .ne. 0) then
          print *,"ERROR in call to: nc_inq_vardimid"
          error_flag = error_netcdf_lib
          return
       end if

       st = nc_inq_varnatts(unit_nc, i-1, var_natts)
       if (st .ne. 0) then
          print *,"ERROR in call to: nc_inq_varnatts"
          error_flag = error_netcdf_lib
          return
       end if

       netcdf_data%variables(i-1)%name  = var_name
       netcdf_data%variables(i-1)%dtype = var_dtype
       netcdf_data%variables(i-1)%ndims = var_ndims
       netcdf_data%variables(i-1)%dimid(0) = var_dimid0
       netcdf_data%variables(i-1)%dimid(1) = var_dimid1
       netcdf_data%variables(i-1)%dimid(2) = var_dimid2
       netcdf_data%variables(i-1)%natts = var_natts
       allocate(netcdf_data%variables(i-1)%attributes(0:var_natts-1), &
                stat=alloc_status)
       if (alloc_status .ne. 0) then
          print *,'ERROR: allocation problem in netcdfreader'
          error_flag = error_allocate
          return
       end if
       
       ! load attributes attached to this variable
       do j=1,var_natts
          call init_attr(netcdf_data%variables(i-1)%attributes(j-1))
          call load_attr(unit_nc, i-1, j-1, &
                         netcdf_data%variables(i-1)%attributes(j-1),error_flag)
          if (error_flag .ne. no_error) stop 1
          if (debug) then
             print *,"DEBUG: loaded attribute: "
             call print_attr(netcdf_data%variables(i-1)%attributes(j-1),&
                             prefix=" DEBUG: loaded attribute")
          end if
       end do
       
       case1d: if (var_ndims .eq. 1) then
          size0 = netcdf_data%dimensions(var_dimid0)%length

          !write(*,*) "var_dtype",var_dtype, nc_type_byte
          
          select case(var_dtype)

          !--- LZ, but nc_type_byte already exist here ----
          case(nc_type_byte) 
             !  #[ handle signed 1 byte integer
             allocate(netcdf_data%variables(i-1)%byte_1d_data(0:size0-1),&
                      stat=alloc_status)
             if (alloc_status .ne. 0) then
                print *,'ERROR: allocation problem in netcdfreader'
                error_flag = error_allocate
                return
             end if
             netcdf_data%variables(i-1)%byte_1d_data(:) = 0
             !write(*,*) 'enter nc_get_var_byte_1d'
             st = nc_get_var_byte_1d(unit_nc, i-1, size0, &
                                      netcdf_data%variables(i-1)%byte_1d_data)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_get_var_byte_1d"
                error_flag = error_netcdf_lib
                return
             end if
             !  #]     

          case(nc_type_short)
             !  #[ handle short type
             allocate(netcdf_data%variables(i-1)%short_1d_data(0:size0-1),&
                      stat=alloc_status)
             if (alloc_status .ne. 0) then
                print *,'ERROR: allocation problem in netcdfreader'
                error_flag = error_allocate
                return
             end if
             netcdf_data%variables(i-1)%short_1d_data(:) = 0
             
             st = nc_get_var_short_1d(unit_nc, i-1, size0, &
                                      netcdf_data%variables(i-1)%short_1d_data)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_get_var_short_1d"
                error_flag = error_netcdf_lib
                return
             end if
             !  #]
          case(nc_type_int)
             !  #[ handle int type
             allocate(netcdf_data%variables(i-1)%int_1d_data(0:size0-1),&
                      stat=alloc_status)
             if (alloc_status .ne. 0) then
                print *,'ERROR: allocation problem in netcdfreader'
                error_flag = error_allocate
                return
             end if
             netcdf_data%variables(i-1)%int_1d_data(:) = 0
             
             st = nc_get_var_int_1d(unit_nc, i-1, size0, &
                                    netcdf_data%variables(i-1)%int_1d_data)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_get_var_int_1d"
                error_flag = error_netcdf_lib
                return
             end if
             !  #]
          case(nc_type_float)
             !  #[ handle float type
             allocate(netcdf_data%variables(i-1)%float_1d_data(0:size0-1),&
                      stat=alloc_status)
             if (alloc_status .ne. 0) then
                print *,'ERROR: allocation problem in netcdfreader'
                error_flag = error_allocate
                return
             end if
             netcdf_data%variables(i-1)%float_1d_data(:) = 0
             
             st = nc_get_var_float_1d(unit_nc, i-1, size0, &
                                      netcdf_data%variables(i-1)%float_1d_data)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_get_var_float_1d"
                error_flag = error_netcdf_lib
                return
             end if
             !  #]
          case(nc_type_double)
             !  #[ handle double type
             allocate(netcdf_data%variables(i-1)%double_1d_data(0:size0-1),&
                      stat=alloc_status)
             if (alloc_status .ne. 0) then
                print *,'ERROR: allocation problem in netcdfreader'
                error_flag = error_allocate
                return
             end if
             netcdf_data%variables(i-1)%double_1d_data(:) = 0
             
             st = nc_get_var_double_1d(unit_nc, i-1, size0, &
                                       netcdf_data%variables(i-1)%double_1d_data)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_get_var_double_1d"
                error_flag = error_netcdf_lib
                return
             end if
             !  #]
          case default
             !  #[ default warning
             print *,'var_name = ['//trim(var_name)//']'
             print *,'Loading 1D arrays is not yet implemented '//&
                     'for var. type: ', var_dtype
             !  #]
          end select
       end if case1d
       
       case2d: if (var_ndims .eq. 2) then
          size0 = netcdf_data%dimensions(var_dimid0)%length
          size1 = netcdf_data%dimensions(var_dimid1)%length

          select case(var_dtype)
          !--- LZ, but nc_type_byte already exist here ----
          case(nc_type_byte) 
             !  #[ handle signed 1 byte integer
             allocate(netcdf_data%variables(i-1)%byte_2d_data(0:size0-1,0:size1-1),&
                      stat=alloc_status)
             if (alloc_status .ne. 0) then
                print *,'ERROR: allocation problem in netcdfreader'
                error_flag = error_allocate
                return
             end if
             netcdf_data%variables(i-1)%byte_2d_data(:,:) = 0
             !write(*,*) 'enter nc_get_var_byte_1d'
             call nc_get_array_byte_2d(unit_nc, i-1, size0, size1, &
                               netcdf_data%variables(i-1)%byte_2d_data, st)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_get_var_byte_2d"
                error_flag = error_netcdf_lib
                return
             end if
             !  #]    


          case(nc_type_char) 
             !  #[ handle ISO/ASII char add by lz
             allocate(netcdf_data%variables(i-1)%char_2d_data(0:size0-1,0:size1-1),&
                      stat=alloc_status)
             if (alloc_status .ne. 0) then
                print *,'ERROR: allocation problem in netcdfreader'
                error_flag = error_allocate
                return
             end if

             netcdf_data%variables(i-1)%char_2d_data(:,:) = ' '

             call nc_get_array_text_2d(unit_nc, i-1, size0, size1, &
                                netcdf_data%variables(i-1)%char_2d_data, st)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_get_var_text_2d"
                error_flag = error_netcdf_lib
                return
             end if
             !  #]  
          !------------------------------------------------------------------
          case(nc_type_short)
             !  #[ handle short type
             allocate(netcdf_data%variables(i-1)%short_2d_data(0:size0-1,0:size1-1),&
                      stat=alloc_status)
             if (alloc_status .ne. 0) then
                print *,'ERROR: allocation problem in netcdfreader'
                error_flag = error_allocate
                return
             end if
             netcdf_data%variables(i-1)%short_2d_data(:,:) = 0
             
             !st = nc_get_var_short_2d(unit_nc, i-1, size0, size1, &
             !                         netcdf_data%variables(i-1)%short_2d_data)
             call nc_get_array_short_2d(unit_nc, i-1, size0, size1, &
                               netcdf_data%variables(i-1)%short_2d_data, st)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_get_var_short_2d"
                error_flag = error_netcdf_lib
                return
             end if
             !  #]
          case(nc_type_int)
             !  #[ handle int type
             allocate(netcdf_data%variables(i-1)%int_2d_data(0:size0-1,0:size1-1),&
                      stat=alloc_status)
             if (alloc_status .ne. 0) then
                print *,'ERROR: allocation problem in netcdfreader'
                error_flag = error_allocate
                return
             end if
             netcdf_data%variables(i-1)%int_2d_data(:,:) = 0
             
             !st = nc_get_var_int_2d(unit_nc, i-1, size0, size1, &
             !                       netcdf_data%variables(i-1)%int_2d_data)
             call nc_get_array_int_2d(unit_nc, i-1, size0, size1, &
                               netcdf_data%variables(i-1)%int_2d_data, st)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_get_var_int_2d"
                error_flag = error_netcdf_lib
                return
             end if
             !  #]
          case(nc_type_float)
             !  #[ handle float type
             allocate(netcdf_data%variables(i-1)%float_2d_data(0:size0-1,0:size1-1),&
                      stat=alloc_status)
             if (alloc_status .ne. 0) then
                print *,'ERROR: allocation problem in netcdfreader'
                error_flag = error_allocate
                return
             end if
             netcdf_data%variables(i-1)%float_2d_data(:,:) = 0
             
             !st = nc_get_var_float_2d(unit_nc, i-1, size0, size1, &
             !                         netcdf_data%variables(i-1)%float_2d_data)

             call nc_get_array_float_2d(unit_nc, i-1, size0, size1, &
                               netcdf_data%variables(i-1)%float_2d_data, st)

             if (st .ne. 0) then
                print *,"ERROR in call to: nc_get_var_float_2d"
                error_flag = error_netcdf_lib
                return
             end if
             !  #]
          case(nc_type_double)
             !  #[ handle double type
             allocate(netcdf_data%variables(i-1)%double_2d_data(0:size0-1,0:size1-1),&
                      stat=alloc_status)
             if (alloc_status .ne. 0) then
                print *,'ERROR: allocation problem in netcdfreader'
                error_flag = error_allocate
                return
             end if
             netcdf_data%variables(i-1)%double_2d_data(:,:) = 0
             
             !st = nc_get_var_double_2d(unit_nc, i-1, size0, size1, &
             !                          netcdf_data%variables(i-1)%double_2d_data)
             call nc_get_array_double_2d(unit_nc, i-1, size0, size1, &
                               netcdf_data%variables(i-1)%double_2d_data, st)

             if (st .ne. 0) then
                print *,"ERROR in call to: nc_get_var_double_2d"
                error_flag = error_netcdf_lib
                return
             end if
             !  #]
          case default
             !  #[ default warning
             print *,'var_name = ['//trim(var_name)//']'
             print *,'Loading 2D arrays is not yet implemented '//&
                     'for var. type: ', var_dtype
             !  #]
          end select
       end if case2d

       case3d: if (var_ndims .eq. 3) then
          size0 = netcdf_data%dimensions(var_dimid0)%length
          size1 = netcdf_data%dimensions(var_dimid1)%length
          size2 = netcdf_data%dimensions(var_dimid2)%length
          
          select case(var_dtype)
          case(nc_type_short)
             !  #[ handle short type
             allocate(netcdf_data%variables(i-1)%&
                      short_3d_data(0:size0-1,0:size1-1,0:size2-1),&
                      stat=alloc_status)
             if (alloc_status .ne. 0) then
                print *,'ERROR: allocation problem in netcdfreader'
                error_flag = error_allocate
                return
             end if
             netcdf_data%variables(i-1)%short_3d_data(:,:,:) = 0
             
             !st = nc_get_var_short_3d(unit_nc, i-1, size0, size1, size2, &
             !                         netcdf_data%variables(i-1)%short_3d_data)
             call nc_get_array_short_3d(unit_nc, i-1, size0, size1, size2, &
                               netcdf_data%variables(i-1)%short_3d_data, st)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_get_var_short_3d"
                error_flag = error_netcdf_lib
                return
             end if
             !  #]
          case(nc_type_int)
             !  #[ handle int type
             allocate(netcdf_data%variables(i-1)%&
                                  int_3d_data(0:size0-1,0:size1-1,0:size2-1),&
                      stat=alloc_status)
             if (alloc_status .ne. 0) then
                print *,'ERROR: allocation problem in netcdfreader'
                error_flag = error_allocate
                return
             end if
             netcdf_data%variables(i-1)%int_3d_data(:,:,:) = 0
             
             !st = nc_get_var_int_3d(unit_nc, i-1, size0, size1, size2, &
             !                       netcdf_data%variables(i-1)%int_3d_data)
             call nc_get_array_int_3d(unit_nc, i-1, size0, size1, size2, &
                               netcdf_data%variables(i-1)%int_3d_data, st)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_get_var_int_3d"
                error_flag = error_netcdf_lib
                return
             end if
             !  #]
          case(nc_type_float)
             !  #[ handle float type
             allocate(netcdf_data%variables(i-1)%&
                                  float_3d_data(0:size0-1,0:size1-1,0:size2-1),&
                      stat=alloc_status)
             if (alloc_status .ne. 0) then
                print *,'ERROR: allocation problem in netcdfreader'
                error_flag = error_allocate
                return
             end if
             netcdf_data%variables(i-1)%float_3d_data(:,:,:) = 0
             
             !st = nc_get_var_float_3d(unit_nc, i-1, size0, size1, size2, &
             !                         netcdf_data%variables(i-1)%float_3d_data)
             call nc_get_array_float_3d(unit_nc, i-1, size0, size1, size2, &
                               netcdf_data%variables(i-1)%float_3d_data, st)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_get_var_float_3d"
                error_flag = error_netcdf_lib
                return
             end if
             !  #]
          case(nc_type_double)
             !  #[ handle double type
             allocate(netcdf_data%variables(i-1)%&
                                  double_3d_data(0:size0-1,0:size1-1,0:size2-1),&
                      stat=alloc_status)
             if (alloc_status .ne. 0) then
                print *,'ERROR: allocation problem in netcdfreader'
                error_flag = error_allocate
                return
             end if
             netcdf_data%variables(i-1)%double_3d_data(:,:,:) = 0
             
             !st = nc_get_var_double_3d(unit_nc, i-1, size0, size1, size2, &
             !                       netcdf_data%variables(i-1)%double_3d_data)
             call nc_get_array_double_3d(unit_nc, i-1, size0, size1, size2, &
                               netcdf_data%variables(i-1)%double_3d_data, st)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_get_var_double_3d"
                error_flag = error_netcdf_lib
                return
             end if
             !  #]
          case default
             !  #[ default warning
             print *,'var_name = ['//trim(var_name)//']'
             print *,'Loading 2D arrays is not yet implemented '//&
                     'for var. type: ', var_dtype
             !  #]
          end select
       end if case3d

       if (debug) then
          call print_var(netcdf_data%variables(i-1))
       end if

    end do varsloop
    
  end subroutine load_all_netcdf_items
    !  #]
  subroutine write_all_netcdf_items(unit_nc, netcdf_data, error_flag)
    !  #[ write all dims, vars and attrs to fil
    integer,                intent(in)  :: unit_nc
    type(netcdf_data_type), intent(in)  :: netcdf_data
    integer,                intent(out) :: error_flag

    ! local variables
    integer :: i, j, st, nc_error_flag, dimid, varid
    integer :: d0, d1, d2, lengths0, lengths1, lengths2
    integer :: start0, start1, start2, ndims
    type(attr_type), pointer :: this_attr

    nullify(this_attr)

    ! write dimensions
    do i=1,netcdf_data%ndims
       
       !print *,"netcdf_data%dimensions(",i-1,")%name = ", &
       !     trim(netcdf_data%dimensions(i-1)%name)
       !print *,"netcdf_data%dimensions(",i-1,")%length = ", &
       !     netcdf_data%dimensions(i-1)%length
       nc_error_flag = nc_def_dim(unit_nc, &
                                  trim(netcdf_data%dimensions(i-1)%name), &
                                  netcdf_data%dimensions(i-1)%length, dimid)
       !print *,'nc_def_dim result: dimid = ', dimid
       if (nc_error_flag .ne. 0) then
          error_flag = error_netcdf_lib
          return
       end if
    end do
    
    ! write global attributes
    gl_attr_loop: do i=1,netcdf_data%natts
       call write_attr(unit_nc, nc_global, &
                       netcdf_data%attributes(i-1), error_flag)
       if (error_flag .ne. no_error) return
    end do gl_attr_loop

    ! define variables
    do i=1,netcdf_data%nvars
       !print *,"writing variable: "//&
       !     trim(netcdf_data%variables(i-1)%name)
       
       select case (netcdf_data%variables(i-1)%dtype)
       case(nc_type_short)
          st = nc_def_var_short(unit_nc, &
                                trim(netcdf_data%variables(i-1)%name), &
                                netcdf_data%variables(i-1)%ndims, &
                                netcdf_data%variables(i-1)%dimid, &
                                varid)
          if (st .ne. 0) then
             print *,"ERROR in call to: nc_def_var_short"
             print *,"for name: "//trim(netcdf_data%variables(i-1)%name)
             error_flag = error_netcdf_lib
             return
          end if
       case(nc_type_int)
          st = nc_def_var_int(unit_nc, &
                              trim(netcdf_data%variables(i-1)%name), &
                              netcdf_data%variables(i-1)%ndims, &
                              netcdf_data%variables(i-1)%dimid, &
                              varid)
          if (st .ne. 0) then
             print *,"ERROR in call to: nc_def_var_int"
             print *,"for name: "//trim(netcdf_data%variables(i-1)%name)
             error_flag = error_netcdf_lib
             return
          end if
       case(nc_type_float)
          st = nc_def_var_float(unit_nc, &
                                trim(netcdf_data%variables(i-1)%name), &
                                netcdf_data%variables(i-1)%ndims, &
                                netcdf_data%variables(i-1)%dimid, &
                                varid)
          if (st .ne. 0) then
             print *,"ERROR in call to: nc_def_var_float"
             print *,"for name: "//trim(netcdf_data%variables(i-1)%name)
             error_flag = error_netcdf_lib
             return
          end if
       case(nc_type_double)
          st = nc_def_var_double(unit_nc, &
                                 trim(netcdf_data%variables(i-1)%name), &
                                 netcdf_data%variables(i-1)%ndims, &
                                 netcdf_data%variables(i-1)%dimid, &
                                 varid)
          if (st .ne. 0) then
             print *,"ERROR in call to: nc_def_var_double"
             print *,"for name: "//trim(netcdf_data%variables(i-1)%name)
             error_flag = error_netcdf_lib
             return
          end if
       case default
          print *,"sorry, writing variables with type "//&
               trim(dtype_to_text(netcdf_data%variables(i-1)%dtype))//&
               " is not yet implemented"
          stop 1
       end select

       ! write the attached attributes
       attr_loop: do j=1,netcdf_data%variables(i-1)%natts
          this_attr => netcdf_data%variables(i-1)%attributes(j-1)
          
          !print *,"debug: writing variable ["//&
          !     trim(netcdf_data%variables(i-1)%name)//"] attribute: "//&
          !     trim(this_attr%name)

          call write_attr(unit_nc, i-1, this_attr, error_flag)
          if (error_flag .ne. no_error) return

       end do attr_loop
       
    end do

    ! leave define mode
    st = nc_enddef(unit_nc)
    if (st .ne. 0) then
       print *,"ERROR in call to: nc_enddef"
       error_flag = error_netcdf_lib
       return
    end if
    
    ! insert data into variables
    varloop: do i=1,netcdf_data%nvars

       ndims = netcdf_data%variables(i-1)%ndims

       d0 = netcdf_data%variables(i-1)%dimid(0)
       d1 = -1
       if (ndims .gt. 1) d1 = netcdf_data%variables(i-1)%dimid(1)
       d2 = -1
       if (ndims .gt. 2) d2 = netcdf_data%variables(i-1)%dimid(2)

       start0 = 0 ! arrays allways start at 0 in the current setup
       start1 = 0
       start2 = 0
       lengths0 = netcdf_data%dimensions(d0)%length
       lengths1 = -1
       if (ndims .gt. 1) lengths1 = netcdf_data%dimensions(d1)%length
       lengths2 = -1
       if (ndims .gt. 2) lengths2 = netcdf_data%dimensions(d2)%length
       
       select case(netcdf_data%variables(i-1)%dtype)
       case(nc_type_short)
          !  #[ write short array of 1, 2 or 3 dim
          select case(netcdf_data%variables(i-1)%ndims)
          case(1)
             call nc_put_array_short_1d(unit_nc, i-1, &
                                        start0, lengths0, &
                             netcdf_data%variables(i-1)%short_1d_data, st)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_put_array_short_1d"
                error_flag = error_netcdf_lib
                return
             end if
          case(2)
             call nc_put_array_short_2d(unit_nc, i-1, &
                             (/ start0, start1 /), &
                             (/ lengths0, lengths1 /), &
                             netcdf_data%variables(i-1)%short_2d_data, st)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_put_array_short_2d"
                error_flag = error_netcdf_lib
                return
             end if
          case(3)
             call nc_put_array_short_3d(unit_nc, i-1, &
                             (/ start0, start1, start2 /), &
                             (/ lengths0, lengths1, lengths2 /), &
                             netcdf_data%variables(i-1)%short_3d_data, st)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_put_array_short_3d"
                error_flag = error_netcdf_lib
                return
             end if
          case default
             print *,"handling data with ",&
                  netcdf_data%variables(i-1)%ndims, &
                  " dimensions is not yet implemented"
          end select ! ndims
          !  #]
       case(nc_type_int)
          !  #[ write int array of 1, 2 or 3 dim
          select case(netcdf_data%variables(i-1)%ndims)
          case(1)
             call nc_put_array_int_1d(unit_nc, i-1, &
                                      start0, lengths0, &
                             netcdf_data%variables(i-1)%int_1d_data, st)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_put_array_int_1d"
                error_flag = error_netcdf_lib
                return
             end if
          case(2)
             call nc_put_array_int_2d(unit_nc, i-1, &
                             (/ start0, start1 /), &
                             (/ lengths0, lengths1 /), &
                             netcdf_data%variables(i-1)%int_2d_data, st)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_put_array_int_2d"
                error_flag = error_netcdf_lib
                return
             end if
          case(3)
             call nc_put_array_int_3d(unit_nc, i-1, &
                             (/ start0, start1, start2 /), &
                             (/ lengths0, lengths1, lengths2 /), &
                             netcdf_data%variables(i-1)%int_3d_data, st)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_put_array_int_3d"
                error_flag = error_netcdf_lib
                return
             end if
          case default
             print *,"handling data with ",&
                  netcdf_data%variables(i-1)%ndims, &
                  " dimensions is not yet implemented"
          end select ! ndims
          !  #]
       case(nc_type_float)
          !  #[ write float array of 1, 2 or 3 dim
          select case(netcdf_data%variables(i-1)%ndims)
          case(1)
             call nc_put_array_float_1d(unit_nc, i-1, &
                                        start0, lengths0, &
                             netcdf_data%variables(i-1)%float_1d_data, st)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_put_array_float_1d"
                error_flag = error_netcdf_lib
                return
             end if
          case(2)
             call nc_put_array_float_2d(unit_nc, i-1, &
                             (/ start0, start1 /), &
                             (/ lengths0, lengths1 /), &
                             netcdf_data%variables(i-1)%float_2d_data, st)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_put_array_float_2d"
                error_flag = error_netcdf_lib
                return
             end if
          case(3)
             call nc_put_array_float_3d(unit_nc, i-1, &
                             (/ start0, start1, start2 /), &
                             (/ lengths0, lengths1, lengths2 /), &
                             netcdf_data%variables(i-1)%float_3d_data, st)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_put_array_float_3d"
                error_flag = error_netcdf_lib
                return
             end if
          case default
             print *,"handling data with ",&
                  netcdf_data%variables(i-1)%ndims, &
                  " dimensions is not yet implemented"
          end select ! ndims
          !  #]
       case(nc_type_double)
          !  #[ write double array of 1, 2 or 3 dim
          select case(netcdf_data%variables(i-1)%ndims)
          case(1)
             call nc_put_array_double_1d(unit_nc, i-1, &
                                         start0, lengths0, &
                             netcdf_data%variables(i-1)%double_1d_data, st)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_put_array_double_1d"
                error_flag = error_netcdf_lib
                return
             end if
          case(2)
             call nc_put_array_double_2d(unit_nc, i-1, &
                             (/ start0, start1 /), &
                             (/ lengths0, lengths1 /), &
                             netcdf_data%variables(i-1)%double_2d_data, st)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_put_array_double_2d"
                error_flag = error_netcdf_lib
                return
             end if
          case(3)
             call nc_put_array_double_3d(unit_nc, i-1, &
                             (/ start0, start1, start2 /), &
                             (/ lengths0, lengths1, lengths2 /), &
                             netcdf_data%variables(i-1)%double_3d_data, st)
             if (st .ne. 0) then
                print *,"ERROR in call to: nc_put_array_double_3d"
                error_flag = error_netcdf_lib
                return
             end if
          case default
             print *,"handling data with ",&
                  netcdf_data%variables(i-1)%ndims, &
                  " dimensions is not yet implemented"
          end select ! ndims
          !  #]
       case default
          print *,"sorry, writing variables with type "//&
               trim(dtype_to_text(netcdf_data%variables(i-1)%dtype))//&
               " is not yet implemented"
          stop 1
       end select ! dtype

    end do varloop

  end subroutine write_all_netcdf_items
    !  #]
  function get_global_attribute(netcdf_data, attr_name) result(attr)
    !  #[ get the attribute structure
    type(netcdf_data_type), intent(in) :: netcdf_data ! input
    character(len=*),       intent(in) :: attr_name   ! input
    type(attr_type)                    :: attr        ! result
    
    ! local variable
    integer :: i

    call init_attr(attr)
    attr_loop: do i=1,netcdf_data%natts
       if (trim(netcdf_data%attributes(i-1)%name) .eq. trim(attr_name)) then
          attr = netcdf_data%attributes(i-1)
          exit attr_loop
       end if
    end do attr_loop

  end function get_global_attribute
    !  #]
  function get_global_text_attribute(netcdf_data, attr_name) result(text_attr)
    !  #[ get text value
    type(netcdf_data_type), intent(in) :: netcdf_data ! input
    character(len=*),       intent(in) :: attr_name   ! input
    character(len=512)                 :: text_attr   ! result
    
    ! local variable
    integer :: i

    text_attr(:) = ' '
    attr_loop: do i=1,netcdf_data%natts
       if (trim(netcdf_data%attributes(i-1)%name) .eq. trim(attr_name)) then
          text_attr = netcdf_data%attributes(i-1)%text
          exit attr_loop
       end if
    end do attr_loop

  end function get_global_text_attribute
    !  #]
  subroutine add_global_attribute(netcdf_data, new_attr, error_flag)
    !  #[ add a new attribute to the data struct
    type(netcdf_data_type), intent(inout) :: netcdf_data
    type(attr_type),        intent(in)    :: new_attr
    integer,                intent(out)   :: error_flag

    ! local variables
    integer :: i, alloc_status, prev_num_atts, new_num_atts
    type(attr_type), dimension(:), pointer :: prev_attributes
    type(attr_type), dimension(:), pointer :: new_attributes
    
    error_flag = no_error

    prev_attributes => netcdf_data%attributes ! could be nullified
    prev_num_atts = netcdf_data%natts         ! could be 0
    new_num_atts = prev_num_atts+1            ! should be 1 or more

    allocate(new_attributes(0:new_num_atts-1), stat=alloc_status)
    if (alloc_status .ne. 0) then
       print *,'ERROR: allocation problem in add_global_attribute'
       error_flag = error_allocate
       return
    end if

    if (new_num_atts .gt. 1) then
       ! an attributes array already exists, so copy the existing data
       do i=1,prev_num_atts
          call copy_attr_content(prev_attributes(i-1), new_attributes(i-1),&
                                 error_flag)
          if (error_flag .ne. no_error) return
       end do
    end if
    
    call copy_attr_content(new_attr, new_attributes(new_num_atts-1), &
                           error_flag)
    if (error_flag .ne. no_error) return

    if (associated(prev_attributes)) deallocate(prev_attributes)

    ! finally add the new attribute
    netcdf_data%natts = new_num_atts
    netcdf_data%attributes => new_attributes

  end subroutine add_global_attribute
    !  #]
  subroutine add_global_text_attr(netcdf_data, attr_name, text, error_flag)
    !  #[
    type(netcdf_data_type), intent(inout) :: netcdf_data
    character(len=*),       intent(in)    :: attr_name, text
    integer,                intent(out)   :: error_flag

    ! local variables
    type(attr_type) :: new_attr

    error_flag = no_error
    call init_attr(new_attr)
    new_attr%name  = trim(attr_name)
    new_attr%dtype = nc_type_char
    new_attr%text  = trim(text)
    new_attr%len   = 1
    call add_global_attribute(netcdf_data, new_attr, error_flag)
    
  end subroutine add_global_text_attr
    !  #]
  subroutine add_global_int_attr(netcdf_data, attr_name, int_val, error_flag)
    !  #[
    type(netcdf_data_type), intent(inout) :: netcdf_data
    character(len=*),       intent(in)    :: attr_name
    integer,                intent(in)    :: int_val
    integer,                intent(out)   :: error_flag

    ! local variables
    type(attr_type) :: new_attr

    error_flag = no_error
    call init_attr(new_attr)
    new_attr%name  = trim(attr_name)
    new_attr%dtype = nc_type_int
    new_attr%int_value = int(int_val, i4_)
    new_attr%len   = 1
    call add_global_attribute(netcdf_data, new_attr, error_flag)
    
  end subroutine add_global_int_attr
    !  #]
  subroutine add_variable(ncdata, new_var, error_flag)
    !  #[ add a new variable
    type(netcdf_data_type), intent(inout) :: ncdata
    type(var_type),         intent(in)    :: new_var
    integer,                intent(out)   :: error_flag
    
    ! local variables
    integer :: i, alloc_status, prev_num_vars, new_num_vars
    type(var_type), dimension(:), pointer :: prev_variables
    type(var_type), dimension(:), pointer :: new_variables

    !select case(l2_var%dtype)

    error_flag = no_error
    
    prev_variables => ncdata%variables ! could be nullified
    prev_num_vars  = ncdata%nvars      ! could be 0
    new_num_vars   = prev_num_vars + 1  ! should be 1 or more
    
    allocate(new_variables(0:new_num_vars-1),stat=alloc_status)
    if (alloc_status .ne. 0) then
       print *,'ERROR: allocation problem in add_1d_dataset'
       error_flag = error_allocate
       return
    end if

    if (new_num_vars .gt. 1) then
       ! one or more variables already exist, so copy them
       do i=0,prev_num_vars-1
          ! may not work with pointers!
          !new_variables(i) = prev_variables(i)
          call move_var_content(prev_variables(i),&  ! source_var
                                new_variables(i)   ) ! dest_var
       end do
    end if

    ! finally add the new variable
    ncdata%nvars = new_num_vars

    new_variables(new_num_vars-1) = new_var
    ncdata%variables => new_variables

    if (associated(prev_variables)) then
       do i=0,prev_num_vars-1
          call delete_var(prev_variables(i))
       end do
       deallocate(prev_variables)
    end if

    ! ncdata%variables(i)%name  = [copy from L2]
    ! ncdata%variables(i)%dtype = [copy from L2]
    ! ncdata%variables(i)%ndims =
    ! ncdata%variables(i)%dimid =
    ! ncdata%variables(i)%natts = [copy from L2 except form coordinates?]
    ! ncdata%variables(i)%attributes(j) [copy from L2 except form coordinates?]
    ! ncdata%variables(i)%short_2d_data => gridded_data_short

  end subroutine add_variable
    !  #]
!  subroutine add_short_dataset() !ncdata, gridded_data_short)!, &
!    ! todo
!  end subroutine add_short_dataset
  subroutine add_int_1d_dataset(ncdata, name, dimid0, int_1d_data, &
                                error_flag)
    !  #[ add a new 1d int variable
    type(netcdf_data_type), intent(inout) :: ncdata
    character(len=*),       intent(in)    :: name
    integer,                intent(in)    :: dimid0
    integer(i4_), dimension(:), pointer   :: int_1d_data
    integer,                intent(out)   :: error_flag

    ! local variables
    type(var_type) :: new_var
    
    error_flag = no_error
    call init_var(new_var)
    new_var%name     = name
    new_var%dtype    = nc_type_int
    new_var%ndims    = 1
    new_var%dimid(0) = dimid0
    new_var%int_1d_data => int_1d_data
    call add_variable(ncdata, new_var, error_flag)

  end subroutine add_int_1d_dataset
    !  #]
  subroutine add_float_1d_dataset(ncdata, name, dimid0, float_1d_data, &
                                  error_flag)
    !  #[ add a new 1d int variable
    type(netcdf_data_type), intent(inout) :: ncdata
    character(len=*),       intent(in)    :: name
    integer,                intent(in)    :: dimid0
    real(r4_), dimension(:), pointer      :: float_1d_data
    integer,                intent(out)   :: error_flag

    ! local variables
    type(var_type) :: new_var
    
    error_flag = no_error
    call init_var(new_var)
    new_var%name     = name
    new_var%dtype    = nc_type_float
    new_var%ndims    = 1
    new_var%dimid(0) = dimid0
    new_var%float_1d_data => float_1d_data
    call add_variable(ncdata, new_var, error_flag)

  end subroutine add_float_1d_dataset
    !  #]
  subroutine add_double_1d_dataset(ncdata, name, dimid0, double_1d_data, &
                                   error_flag)
    !  #[ add a new 1d double variable
    type(netcdf_data_type), intent(inout) :: ncdata
    character(len=*),       intent(in)    :: name
    integer,                intent(in)    :: dimid0
    real(r8_), dimension(:), pointer      :: double_1d_data
    integer,                intent(out)   :: error_flag

    ! local variables
    type(var_type) :: new_var
    
    error_flag = no_error
    call init_var(new_var)
    new_var%name     = name
    new_var%dtype    = nc_type_double
    new_var%ndims    = 1
    new_var%dimid(0) = dimid0
    new_var%double_1d_data => double_1d_data
    call add_variable(ncdata, new_var, error_flag)

  end subroutine add_double_1d_dataset
    !  #]
  function get_varid_by_name(ncdata, name) result(varid)
    !  #[ get a varid
    type(netcdf_data_type), intent(in) :: ncdata
    character(len=*),       intent(in) :: name
    integer                            :: varid ! result

    ! local variable
    integer :: i

    varid=-1
    searchloop: do i=0, ncdata%nvars-1
       if (trim(ncdata%variables(i)%name) .eq. trim(name)) then
          varid=i
          exit searchloop
       end if
    end do searchloop

    if (varid .eq. -1) then
       print *,"ERROR in get_varid_by_name"
       print *,"Could not find variable with name: "//trim(name)
       stop 1
    end if

  end function get_varid_by_name
    !  #]
  !  #]
  !  #[ some higher level composite subroutines
  subroutine nc_create_var_short_2d(var_name, ndims, dimids, standard_name, &
       long_name, units, scale_factor, add_offset, valid_min, valid_max, &
       coordinates, extra_attr, short_2d_values, ncdata, error_flag)
    !  #[ replacement for similar subroutine in NetCDFMod.F90
    !
    ! Description: create int NetCDF variable with some standard settings
    !
    character(len=*), intent(in)  :: var_name
    integer,          intent(in)  :: ndims
    integer, dimension(0:ndims-1), intent(in) :: dimids
    character(len=*), optional, intent(in)  :: standard_name
    character(len=*), optional, intent(in)  :: long_name
    character(len=*), optional, intent(in)  :: units
    real(r8_),        optional, intent(in)  :: scale_factor
    real(r8_),        optional, intent(in)  :: add_offset
    real(r8_),        optional, intent(in)  :: valid_min
    real(r8_),        optional, intent(in)  :: valid_max
    character(len=*), optional, intent(in)  :: coordinates
    character(len=*), optional, intent(in)  :: extra_attr
    integer(i2_), dimension(:,:), pointer :: short_2d_values
    type(netcdf_data_type), intent(inout) :: ncdata
    integer,          intent(out) :: error_flag

    ! local variables
    type(var_type) :: variable
    integer(i2_), dimension(:,:), pointer :: short_2d_copied_values
    integer :: size1, size2, status
    
    call init_var(variable)
    variable%name  = var_name
    variable%dtype = nc_type_short
    variable%ndims = ndims
    variable%dimid(0:1) = dimids

    call nc_add_standard_attrs(variable, standard_name, &
         long_name, units, scale_factor, add_offset, valid_min, valid_max, &
         coordinates, extra_attr, nc_type_short, error_flag)
      
    ! possibly some extra checks could be added here
    ! to ensure the 2d array size matches with the specified
    ! netcdf dimensions

    ! finally, copy the array of input data, to make sure we get
    ! no segmentation faults if the user deallocated the array
    ! before doing the actual write to the netcdf file...
    size1 = size(short_2d_values, 1)
    size2 = size(short_2d_values, 2)
    allocate(short_2d_copied_values(size1, size2),stat=status)
    if (status .ne. 0) then
       error_flag = error_allocate
       return
    end if

    short_2d_copied_values(:,:) = short_2d_values(:,:)

    variable%short_2d_data => short_2d_copied_values

    call add_variable(ncdata, variable, error_flag)
    if (error_flag .ne. no_error) return
  
  end subroutine nc_create_var_short_2d
  !  #]
  subroutine nc_create_var_int_2d(var_name, ndims, dimids, standard_name, &
       long_name, units, scale_factor, add_offset, valid_min, valid_max, &
       coordinates, extra_attr, int_2d_values, ncdata,  error_flag)
    !  #[ replacement for similar subroutine in NetCDFMod.F90
    !
    ! Description: create int NetCDF variable with some standard settings
    !
    character(len=*), intent(in)  :: var_name
    integer,          intent(in)  :: ndims
    integer, dimension(0:ndims-1), intent(in) :: dimids
    character(len=*), optional, intent(in)  :: standard_name
    character(len=*), optional, intent(in)  :: long_name
    character(len=*), optional, intent(in)  :: units
    real(r8_),        optional, intent(in)  :: scale_factor
    real(r8_),        optional, intent(in)  :: add_offset
    real(r8_),        optional, intent(in)  :: valid_min
    real(r8_),        optional, intent(in)  :: valid_max
    character(len=*), optional, intent(in)  :: coordinates
    character(len=*), optional, intent(in)  :: extra_attr
    integer(i4_), dimension(:,:), pointer :: int_2d_values
    type(netcdf_data_type), intent(inout) :: ncdata
    integer,          intent(out) :: error_flag

    ! local variables
    type(var_type) :: variable
    integer(i4_), dimension(:,:), pointer :: int_2d_copied_values
    integer :: size1, size2, status

    call init_var(variable)
    variable%name  = var_name
    variable%dtype = nc_type_int
    variable%ndims = ndims
    variable%dimid(0:1) = dimids
    call nc_add_standard_attrs(variable, standard_name, &
         long_name, units, scale_factor, add_offset, valid_min, valid_max, &
         coordinates, extra_attr, nc_type_int, error_flag)
      
    ! possibly some extra checks could be added here
    ! to ensure the 2d array size matches with the specified
    ! netcdf dimensions

    ! finally, copy the array of input data, to make sure we get
    ! no segmentation faults if the user deallocated the array
    ! before doing the actual write to the netcdf file...
    size1 = size(int_2d_values, 1)
    size2 = size(int_2d_values, 2)
    allocate(int_2d_copied_values(size1, size2),stat=status)
    if (status .ne. 0) then
       error_flag = error_allocate
       return
    end if

    int_2d_copied_values(:,:) =  int_2d_values(:,:)

    variable%int_2d_data => int_2d_copied_values

    call add_variable(ncdata, variable, error_flag)
    if (error_flag .ne. no_error) return
  
  end subroutine nc_create_var_int_2d
  !  #]
  subroutine nc_add_standard_attrs(variable, standard_name, &
       long_name, units, scale_factor, add_offset, valid_min, valid_max, &
       coordinates, extra_attr, dtype, error_flag)
    !  #[ define some standard attributes
    type(var_type),             intent(inout) :: variable
    character(len=*), optional, intent(in)    :: standard_name
    character(len=*), optional, intent(in)    :: long_name
    character(len=*), optional, intent(in)    :: units
    real(r8_),        optional, intent(in)    :: scale_factor
    real(r8_),        optional, intent(in)    :: add_offset
    real(r8_),        optional, intent(in)    :: valid_min
    real(r8_),        optional, intent(in)    :: valid_max
    character(len=*), optional, intent(in)    :: coordinates
    character(len=*), optional, intent(in)    :: extra_attr
    integer,                    intent(in)    :: dtype
    integer,                    intent(out)   :: error_flag

    ! local variables
    real(r8_) :: add_offset_to_use
    real(r8_) :: scale_factor_to_use
    character(len=2)   :: separator
    character(len=75)  :: extra_attr_name
    character(len=200) :: extra_attr_value
    integer :: pos
    
    if (dtype .eq. nc_type_short) then
       call add_short_attr(variable, '_FillValue', undef_short, error_flag)
       if (error_flag .ne. no_error) return
       call add_short_attr(variable, 'missing_value', undef_short, error_flag)
       if (error_flag .ne. no_error) return
    else if (dtype .eq. nc_type_int) then
       call add_int_attr(variable, '_FillValue', undef_int, error_flag)
       if (error_flag .ne. no_error) return
       call add_int_attr(variable, 'missing_value', undef_int, error_flag)
       if (error_flag .ne. no_error) return
    else if (dtype .eq. nc_type_float) then
       call add_float_attr(variable, '_FillValue', undef_float, error_flag)
       if (error_flag .ne. no_error) return
       call add_float_attr(variable, 'missing_value', undef_float, error_flag)
       if (error_flag .ne. no_error) return
    else if (dtype .eq. nc_type_double) then
       call add_double_attr(variable, '_FillValue', undef_double, error_flag)
       if (error_flag .ne. no_error) return
       call add_double_attr(variable, 'missing_value', undef_double, error_flag)
       if (error_flag .ne. no_error) return
    end if

    add_offset_to_use = 0._R8_
    if (present(add_offset)) add_offset_to_use = add_offset

    scale_factor_to_use = 1._r8_
    if (present(scale_factor)) scale_factor_to_use = scale_factor
    
    ! apply scale factor and offset to min and max values
    ! if both attributes are present, the data are scaled before the offset
    ! is added when scaled data are written, the application should first
    ! subtract the offset and then divide by the scale factor
    if (present(valid_min)) then
       if (dtype .eq. nc_type_short) then
          call add_short_attr(variable, 'valid_min', &
               nint( (valid_min-add_offset_to_use) / &
                     scale_factor_to_use,              i2_), &
               error_flag)
          if (error_flag .ne. no_error) return
       else if (dtype .eq. nc_type_int) then
          call add_int_attr(variable, 'valid_min', &
               nint( (valid_min-add_offset_to_use) / &
                     scale_factor_to_use,              i4_), &
               error_flag)
          if (error_flag .ne. no_error) return
       else
          print *,'ERROR in nc_add_standard_attrs: valid_max attribute'
          print *,'not yet implemented for this datatype: ', dtype
          error_flag = error_not_yet_implemented
          return
       end if
    end if

    if (present(valid_max)) then
       if (dtype .eq. nc_type_short) then
          call add_short_attr(variable, 'valid_max', &
               nint( (valid_max-add_offset_to_use) / &
                     scale_factor_to_use,             i2_), &
               error_flag)
          if (error_flag .ne. no_error) return
       else if (dtype .eq. nc_type_int) then
          call add_int_attr(variable, 'valid_max', &
               nint( (valid_max-add_offset_to_use) / &
                     scale_factor_to_use,             i4_), &
               error_flag)
          if (error_flag .ne. no_error) return
       else
          print *,'ERROR in nc_add_standard_attrs: valid_max attribute'
          print *,'not yet implemented for this datatype: ', dtype
          error_flag = error_not_yet_implemented
          return
       end if
    end if
    
    if (present(standard_name)) then
       if (len_trim(standard_name) .gt. 0) then
          call add_text_attr(variable, 'standard_name', standard_name, &
                             error_flag)
          if (error_flag .ne. no_error) return
       endif
    endif

    if (present(long_name)) then
       call add_text_attr(variable, 'long_name', long_name, error_flag)
       if (error_flag .ne. no_error) return
    end if

    if (present(units)) then
       if (len_trim(units) .gt. 0) then
          call add_text_attr(variable, 'units', units, error_flag)
          if (error_flag .ne. no_error) return
       endif
    endif

    ! create scale factor and offset attributes
    ! if one of them differs from 1 resp. 0
    if (present(scale_factor) .or. present(add_offset)) then
!       if ( (abs(scale_factor_to_use - 1.0) .gt. 1.e-10) .or. &
!            (abs(add_offset_to_use)         .gt. 1.e-10)      ) then
          call add_double_attr(variable, 'scale_factor', scale_factor_to_use, &
                               error_flag)
          if (error_flag .ne. no_error) return

          call add_double_attr(variable, 'add_offset', add_offset_to_use,&
                               error_flag)
          if (error_flag .ne. no_error) return
!       end if
    endif

    if (present(coordinates)) then
       if (len_trim(coordinates) .gt. 0) then
          call add_text_attr(variable, 'coordinates', coordinates, error_flag)
          if (error_flag .ne. no_error) return
       end if
    end if

    if (present(extra_attr)) then
       ! for now, assume this is always a text attribute
       separator = '::'
       pos = index(extra_attr, separator)

       if (pos .eq. 0) then
          print *,"ERROR in nc_add_standard_attrs: "
          print *,"ERROR: expected a separator: "//trim(separator)
          print *,"but could not find it in the extra_attr definition"
          stop 1
       end if

       if (pos .gt. len_trim(extra_attr)-len_trim(separator)) then
          print *,"ERROR in nc_add_standard_attrs: "
          print *,"ERROR: pos valid seems invalid"
          stop 1
       end if
       
       extra_attr_name  = extra_attr(:pos-1)
       extra_attr_value = extra_attr(pos+len_trim(separator):)

       call add_text_attr(variable, extra_attr_name, extra_attr_value, &
                          error_flag)
       if (error_flag .ne. no_error) return
       
    end if
    
  end subroutine nc_add_standard_attrs
    !  #]
  !  #]
  !  #[ file handling
  subroutine read_netcdf_file(filename_nc_in, netcdf_data, error_flag)
    !  #[ read from file
    character(len=*), intent(in) :: filename_nc_in
    type(netcdf_data_type), intent(out) :: netcdf_data
    integer, intent(out) :: error_flag

    ! local variables
    integer :: st, unit_nc
    logical :: fexist
    
    ! init
    error_flag = no_error

    inquire(file=filename_nc_in, exist=fexist)
    if (.not. fexist) then
       print *,'Sorry, cannot find input file: '//trim(filename_nc_in)
       error_flag = error_opening_file
       return
    end if
    
    st = nc_open(filename_nc_in, 'r', unit_nc)
    if (st .ne. 0) then
       print *,"ERROR in call to: nc_open"
       error_flag = error_netcdf_lib
       return
    end if

    call load_all_netcdf_items(unit_nc, netcdf_data, error_flag)
    if (error_flag .ne. no_error) return
    
    ! close the file
    st = nc_close(unit_nc)
    if (st .ne. 0) then
       print *,"ERROR in call to: nc_close"
       error_flag = error_closing_file
       return
    end if
     
  end subroutine read_netcdf_file
    !  #]
  subroutine write_netcdf_file(filename_nc_out, netcdf_data, error_flag)
    !  #[ write to file
    character(len=*), intent(in) :: filename_nc_out
    type(netcdf_data_type), intent(in) :: netcdf_data
    integer, intent(out) :: error_flag

    ! local variables
    integer :: st, unit_nc
    logical :: fexist
    
    ! init
    error_flag = no_error

    inquire(file=filename_nc_out, exist=fexist)
    if (fexist) then
       print *,'Sorry, output file: '//trim(filename_nc_out)
       print *,"already exists. I will not overwrite it."
       error_flag = error_opening_file
       return
    end if

    print *,"writing to file: "//trim(filename_nc_out)

    st = nc_create(trim(filename_nc_out) // achar(0), unit_nc)
    if (st .ne. 0) then
       print *,"ERROR in call to: nc_create"
       error_flag = error_netcdf_lib
       return
    end if
    
    call write_all_netcdf_items(unit_nc, netcdf_data, error_flag)
    if (error_flag .ne. no_error) return
    
    ! close the file
    st = nc_close(unit_nc)
    if (st .ne. 0) then
       print *,"ERROR in call to: nc_close"
       error_flag = error_closing_file
       return
    end if

  end subroutine write_netcdf_file
    !  #]
  subroutine check_c_f90_interface
    !  #[ check the c f90 interface
    integer(i1_) :: size_short
    integer(i1_) :: size_int
!    integer(i1_) :: size_long
    integer(i1_) :: size_float
    integer(i1_) :: size_double

    integer      :: default_integer
    integer      :: nbytes_default_integer
    
    size_short  = get_bytesize_short()
    size_int    = get_bytesize_int()
    !size_long   = get_bytesize_long()
    size_float  = get_bytesize_float()
    size_double = get_bytesize_double()
    
    inquire(iolength=nbytes_default_integer) default_integer  

    ! JK: the long check disabled for now since it fails on 32-bit machines.
    ! Currently we don't use the long int type in c in our interfacing
    ! so it is not relevant.

    if ( (size_short  .ne. 2) .or. &
         (size_int    .ne. 4) .or. &
!         (size_long   .ne. 8) .or. &
         (size_float  .ne. 4) .or. &
         (size_double .ne. 8)      ) then
       print *,"ERROR: variable sizes are not correct."
       print *,"This netcdf reader expects the following settings: "
       print "(A,i1,A)","c: size_short  = 2 (actual size is ", size_short,")"
       print "(A,i1,A)","c: size_int    = 4 (actual size is ", size_int,")"
!       print "(A,i1,A)","c: size_long   = 8 (actual size is ", size_long,")"
       print "(A,i1,A)","c: size_float  = 4 (actual size is ", size_float,")"
       print "(A,i1,A)","c: size_double = 8 (actual size is ", size_double,")"
       print *,"Please adapt your compiler options to get matching sizes"
       print *,"otherwise the fortran code cannot commmunicate with the c code"

       print *,"fortran: nbytes_default_integer = ", nbytes_default_integer
       
       stop 1
    end if

  end subroutine check_c_f90_interface
    !  #]
  !  #]
  !---------------------------
end module NetcdfReaderWriter
