program test_bufr_data_extraction_module

  use BufrIO, only: set_BUFR_fileattributes, get_BUFR_nr_of_messages, &
                    open_BUFR_file, get_BUFR_message, close_BUFR_file, &
                    bufr_file_attr_data, bufr_data_type
  use bufr_data_extraction_module
  use Compiler_Features, only: getarg_genscat, iargc_genscat
  
  character(len=256) :: filename_bufr
  type(bufr_file_attr_data) :: bufr_file_attributes
  type(bufr_data_type) :: tbd
  integer :: nr_of_messages, kmsg, ksub, error_flag
  
  ! read command line
  narg = iargc_genscat()
  if (narg .lt. 1) then
     call print_usage()
     stop
  endif

  ! the first argument defines the bufr input filename
  call getarg_genscat(1,filename_bufr)
  ! filename_bufr = "test_data/QS100_D09014_S2331_E0043_B4986162"

  call set_BUFR_fileattributes(filename_bufr, bufr_file_attributes, error_flag)
  call open_BUFR_file(bufr_file_attributes, error_flag)

  nr_of_messages = get_BUFR_nr_of_messages(bufr_file_attributes)

  ! read first BUFR message and retrieve some information from it
  call get_BUFR_message(bufr_file_attributes, 1, tbd, error_flag)

  ! choose which BUFR template to use when interpreting the data
  ! (i.e. determine type of message depending on BUFR descriptor)
  ! this assumes all bufr messages in the file use the same template.
  call set_BUFR_template(tbd)

  ! ensure wind direction angles are reported using
  ! oceanographic angle convention by the BUFR routines
  call report_angle_conv_oceanographic()

  msgloop:do kmsg= 1, nr_of_messages
     ! import data from bufr message
     call get_BUFR_message(bufr_file_attributes, kmsg, tbd, error_flag)
     ! loop over Wind Vector Cells in BUFR message
     subset_loop: do ksub = 1, tbd%nsubsets
        ! pointer to the position just before the start of this cell
        call goto_subset(tbd,ksub)
        if (.not. missing_real(get_sigma0(tbd, 1))) then
           print *,"results for message: ", kmsg," and subset: ", ksub
           print *,'get_mspd(tbd) = ', get_mspd(tbd)
           ! get inc, azi and sigma0 for all 4 beams
           print *,'incident angles: ', &
                get_inc_angle(tbd,1), get_inc_angle(tbd,2), &
                get_inc_angle(tbd,3), get_inc_angle(tbd,4)," [deg]"
           print *,'azimuth angles: ', &
                get_azi_angle(tbd,1), get_azi_angle(tbd,2), &
                get_azi_angle(tbd,3), get_azi_angle(tbd,4)," [deg]"
           print *,"sigma0 values: ", &
                get_sigma0(tbd, 1), get_sigma0(tbd, 2), &
                get_sigma0(tbd, 3), get_sigma0(tbd, 4), " [dB]"
           print *,"polarisation: ", &
                get_polarisation(tbd, 1), get_polarisation(tbd, 2), &
                get_polarisation(tbd, 3), get_polarisation(tbd, 4), &
                " [0=HH, 1=VV]"
        end if
        !if (ksub .gt. 5) exit subset_loop
     end do subset_loop
     !if (kmsg .gt. 2) exit msgloop
  end do msgloop

  ! close input file
  call close_BUFR_file(bufr_file_attributes, error_flag)
contains
  !----------------------
  subroutine print_usage()
    !  #[ print some help text
    print *,' '
    print *,'Usage: test_bufr_data_extraction_module <BUFR input file>'
    print *,' '
  end subroutine print_usage
  !----------------------
end program test_bufr_data_extraction_module
